#ifdef REAL
#  define SDOT  ddot
#  define SNRM2 dnrm2
#else
#  define SDOT  sdot
#  define SNRM2 snrm2
#endif

      subroutine cgopt( nn, xx, ff, gg, fcgrd, optline
     $                  , epsx, fmin, epsg
     $                  , iprint, niter, nfunc, rz, nrz
     $                  , ifail )

      implicit none

c-----------------------------------------
c declare arguments
c-----------------------------------------
      integer nn, iprint, niter, nfunc, nrz, ifail
      REAL    xx(nn), ff, gg(nn), epsx, fmin, epsg, rz(nrz)

      external fcgrd, optline

c-----------------------------------------
C declare local variables
c-----------------------------------------
      external SNRM2
      REAL     SNRM2

c-----------------------------------------
c print information
c-----------------------------------------
      if (iprint .ge. 1) then
         print '(a)', '==============================================='
         print '(a)', '===         O P T I M I Z A T I O N         ==='
         print '(a)', '===   C O N J U G A T E  G R A D I E N T S  ==='
         print '(a)', '===      B Y  P O L A K - R I B I E R E     ==='
         print '(a)', '==============================================='
         print '(a,i9)'
     $        , '  number of control variables.......', nn
         print '(a,e9.2)'
     $        , '  precision on x....................', epsx
         print '(a,e9.2)'
     $        , '  precision on g....................', epsg
         print '(a,e9.2)'
     $        , '  expected optimal function value...', fmin
         print '(a,i9)'
     $        , '  maximal number of iterations......', niter
         print '(a,i9)'
     $        , '  maximal number of simulations.....', nfunc
         print '(a,i9)'
     $        , '  information level.................', iprint
         print '(a,i9)'
     $        , '  size of allocated memory..........', nrz
         print '(a,i9)'
     $        , '  size of used memory...............', 3*nn
         print *
      endif

c-----------------------------------------
c check arguments
c-----------------------------------------
      ifail = 0

      if (nn .le. 0) then
         if (iprint.ge.1) then
            print '(a,i6)'  , '  ERROR : n     = ', nn
         endif
         ifail = 1
         goto 999
      endif

      if (niter .le. 0) then
         if (iprint.ge.1) then
            print '(a,i6)'  , '  ERROR : niter = ', niter
         endif
         ifail = 1
         goto 999
      endif

      if (nfunc .le. 0) then
         if (iprint.ge.10) then
            print '(a,i6)'  , '  ERROR : nfunc  = ', nfunc
         endif
         ifail = 1
         goto 999
      endif

      if (epsx .le. 0.) then
         if (iprint.ge.1) then
            print '(a,e9.2)', '  ERROR : epsx = ', epsx
         endif
         ifail = 1
         goto 999
      endif

      if (epsg .le. 0.) then
         if (iprint.ge.1) then
            print '(a,e9.2)', '  ERROR : epsg  = ', epsg
         endif
         ifail = 1
         goto 999
      endif

      if (epsg .gt. 1.) then
         if (iprint.ge.1) then
            print '(a,e9.2)', '  ERROR : epsg  = ', epsg
         endif
         ifail = 1
         goto 999
      endif

      if (nrz .lt. 3*nn) then
         if (iprint.ge.1) then
            print '(a)', ' ERROR : not enough memory allocated'
         endif
         ifail = 1
         goto 999
      endif

c-----------------------------------------
c call the optimization code
c-----------------------------------------
      call cgoptw( nn, xx, ff, gg, fcgrd, optline
     $            , epsx, fmin, epsg, iprint
     $            , niter, nfunc, ifail
     $            , rz(1), rz(1+nn), rz(1+nn+nn)
     $            )


c-----------------------------------------
c print final information
c-----------------------------------------
      if (iprint .ge. 5) then
         print *
         print '(a,i9)'
     $        , '  number of iterations..............', niter
         print '(a,i9)'
     $        , '  number of simultations............', nfunc
         print '(a,e9.2)'
     $        , '  relative precision on g...........', epsg
      end if

      if (iprint.ge.10) then
         print *
         print '(a,e15.8)'
     $        , '  cost function...............', ff
         print '(a,e15.8)'
     $        , '  norm of x...................', SNRM2( nn, xx )
         print '(a,e15.8)'
     $        , '  norm of g...................', SNRM2( nn, gg )
      end if

c-----------------------------------------
c print error message
c-----------------------------------------
 999  continue

      if (ifail .ne. 0) then
         if (iprint .ge. 5) then
            print *
            print '(a)', '  optimization stopped because :'
            if (ifail .lt. 0) then
               print '(a)'
     $              , '    user request during simul'
            else if (ifail .eq. 0) then
               print '(a)'
     $              , '    optimal solution found'
            else if (ifail .eq. 1) then
               print '(a)'
     $              , '    an input argument is wrong'
            else if (ifail .eq. 3) then
               print '(a)'
     $              , '    the initial gradient is too small'
            else if (ifail .eq. 4) then
               print '(a)'
     $              , '    the search direction is not a descent one'
            else if (ifail .eq. 5) then
               print '(a)'
     $              , '    maximal number of iterations reached'
            else if (ifail .eq. 6) then
               print '(a)'
     $              , '    maximal number of simulations reached'
            else if (ifail .eq. 7) then
               print '(a)'
     $              , '    the linesearch failed'
            else if (ifail .eq. 8) then
               print '(a)'
     $              , '    the function could not be improved'
            end if
            print *
         else if (iprint .ge. 1) then
            print *
            print '(a,i9)'
     $           , '  after optimization ifail..........', ifail
            print *
         end if
      end if

c-----------------------------------------
c end of subroutine
c-----------------------------------------
      return
      end
