#ifdef REAL
#  define SDOT  ddot
#  define SNRM2 dnrm2
#else
#  define SDOT  sdot
#  define SNRM2 snrm2
#endif

      subroutine lsline( n, simul, xn, fn, fpn, t, tmin, tmax, d, g
     1                  , amd, amf, imp, logic, nap, napmax, x )

      implicit none

c----------------------------------
c declare arguments
c----------------------------------
      integer n, imp, logic, nap, napmax
      REAL    xn(n), fn, fpn, t, tmin, tmax, d(n), g(n), amd, amf, x(n)
      external simul

c----------------------------------
c declare local variables
c----------------------------------
      REAL    factor
      parameter( factor = 0.2 )

      REAL    barmax
      parameter( barmax = 0.3 )
      REAL    barmul
      parameter( barmul = 5.0 )
      REAL    barmin
      parameter( barmin = 0.01 )

      integer i, indic, indica, indicd, iter
      REAL    tesf, tesd, tg, fg, td, ta
      REAL    fa, fpa, f, fp, ffn
      REAL    z, test, barr
      REAL    left, right, told

      external SDOT
      REAL     SDOT

c----------------------------------
c check arguments
c----------------------------------
      if (  (n.le.0)
     $ .or. (fpn.ge.0.0)  .or. (t.le.0.0)
     $ .or. (tmax.le.0.0) .or. (amf.le.0.0)
     $ .or. (amd.le.amf)  .or. (amd.ge.1.0) ) then
        logic = 6
        goto 999
      endif

c----------------------------------
c initialization
c----------------------------------
      tesf   = amf*fpn
      tesd   = amd*fpn
      barr   = barmin
      td     = 0.0
      tg     = 0.0
      fg     = fn
      ta     = 0.0
      fa     = fn
      fpa    = fpn

c----------------------------------
c make sure that t is between
c tmin and tmax
c----------------------------------
      if (t.le.tmin) then
         t = tmin
         if (t.gt.tmax) then
            tmin = tmax
         endif
      endif

   20 continue
      if (fn+t*fpn .ge. fn+0.9*t*fpn) then
         t = 2.*t
         goto 20
      endif

      indica = 1
      logic  = 0
      if (t .gt. tmax) then
          t = tmax
          logic = 1
      endif

c----------------------------------
c compute new x
c----------------------------------
      do i = 1,n
         x(i) = xn(i) + t*d(i)
      end do

c----------------------------------
c begin of iteration
c----------------------------------
      do iter = nap+1, napmax

      indic = 4

c------------------------------------
c compute cost function and gradient
c------------------------------------
      call simul( indic, n, x, f, g )

c---------------------------
c stop on user request
c---------------------------
      if (indic .eq. 0) then
         logic = 5
         fn    = f
         do i = 1,n
            xn(i) = x(i)
         end do
         goto 999
      endif

c-----------------------------------------
c does the caculation of simul has been    
c succesfull ?
c-----------------------------------------
      if (indic .lt. 0) then
         td     = t
         indicd = indic
         logic  = 0
         t = tg + factor*(td-tg)
         goto 905
      endif

c-----------------------------------------
c check the general mathematical conditions 
c for computing
c-----------------------------------------
      fp = SDOT( n, d, g )

c-----------------------------------------
c apply Wolfe test
c-----------------------------------------
      ffn = f-fn
      if (ffn .gt. t*tesf) then
         td     = t
         indicd = indic
         logic  = 0
         goto 500
      endif

c----------------------------------------- 
c if first Wolfe test ok, try second 
c-----------------------------------------
      if (fp .gt. tesd) then
         logic = 0
         goto 320
      endif

      if (logic.eq.0) goto 350

c-----------------------------------------
c if second Wolfe test fails, exit
c-----------------------------------------
 320  continue
      fn = f
      do i = 1,n
         xn(i) = x(i)
      end do
      goto 999


 350  continue
      tg  = t
      fg  = f
      if (td .ne. 0.0) goto 500

c-----------------------------------------
c extrapolation
c-----------------------------------------
      told    = t
      left = (1.0+barmin)*t
      right = 10.0*t
      call cubic( t, f, fp, ta, fa, fpa, left, right )
      ta     = told
      if (t.ge.tmax) then
         logic = 1
         t     = tmax
      endif
      goto 900

c-----------------------------------------
c interpolation
c-----------------------------------------
 500  continue
      if (indica.le.0) then
         ta = t
         t  = (1.0-factor)*tg + factor*td
         goto 900
      endif
      test   = barr*(td-tg)
      left = tg+test
      right = td-test
      told    = t
      call cubic( t, f, fp, ta, fa, fpa, left, right )
      ta = told
      if (t.gt.left .and. t.lt.right) then
         barr = max( barmin, barr/barmul )
      else
         barr = min( barmul*barr, barmax )
      endif

c-----------------------------------------
c end of iteration loop
c-----------------------------------------
 900  continue
      fa  = f
      fpa = fp

 905  continue
      indica = indic

c-----------------------------------------
c continue ?
c-----------------------------------------
      if (td .eq. 0.0)     goto 950
      if (td-tg .lt. tmin) goto 920

c-----------------------------------------
c test if maschine precision is reached
c-----------------------------------------
      do i = 1,n
         z = xn(i) + t*d(i)
         if ((z.ne.xn(i)) .and. (z.ne.x(i))) goto 950
      end do

 920  continue
      logic = 6

c-----------------------------------------
c if indicd < 0, the interpolation/extrapolation
c hasn't been run  
c-----------------------------------------
      if (indicd .lt. 0) logic = indicd

c-----------------------------------------
c if tg=0, xn = xn_depart, use xn=x_left 
c which decreases  f
c-----------------------------------------
      if (tg .ne. 0.0) then
         fn = fg
         do i = 1,n
            xn(i) = xn(i) + tg*d(i)
         end do
      endif
      goto 999

c-----------------------------------------
c update vector for new iteration
c-----------------------------------------
 950  continue

      do i = 1,n
         x(i) = xn(i) + t*d(i)
      end do

c-----------------------------------------
c end of iteration
c-----------------------------------------
      end do

c-----------------------------------------
c to many function evaluations
c-----------------------------------------
      logic = 4
      fn    = fg
      do i = 1,n
         xn(i) = xn(i) + tg*d(i)
      end do

c-----------------------------------------
c end of routine
c-----------------------------------------
  999 continue
      nap = iter

      return
      end
