C******************************************************************
C******************************************************************
      subroutine cmesh (dx, apb, amb, wid, n, ndir)
      implicit none

      REAL     dx(1000)
      REAL     apb, amb, wid
      integer  n, ndir

      REAL     pi, epsln, a, b, c, rn, frac
      integer  i, ii

      pi    = 3.1415926
      epsln = 0.001
      a     = 0.5*(apb+amb)
      b     = 0.5*(apb-amb)
      c     = (wid/a) + epsln
      n     = c
      rn    = 1./float(n)
      do ii = 1,n
         i=ii
         if (ndir .eq. 1)  i = n+1-ii
         dx(i) = a + b*cos(pi*rn*(ii-0.5))
      end do
      write (7,98) apb, amb, wid, c
      frac = c-int(c)
      if (frac .gt. 0.01*a) then
         write (7, 99)
         stop ' problem in cmesh'
      endif

 98   format (/1x,'max res=',e9.4,'  min res=',e9.4,'  width=',e9.4,
     *     '  number spaces=',e10.7/)
 99   format (/1x,' !!! non integral number of spacings in "wid" !!!'/)

      end


C******************************************************************
C******************************************************************
      subroutine mesh (dx, dy)
      implicit none

      integer    im   , jm   , mx      , nxres  , nyres
      parameter (im=18, jm=30, mx=im*jm, nxres=1, nyres=1)
      integer    imt   , jmt   , km  , itte
      parameter (imt=im, jmt=jm, km=1, itte=100)

      REAL     boxtw, boxts, xt(imt), yt(jmt), xv(imt), yv(jmt)
      REAL     deltaz(km)
      common /coord/ boxtw, boxts, xt, yt, xv, yv, deltaz

      integer idir(nxres), jdir(nyres)
      REAL    xmax(nxres), xmin(nxres), xwid(nxres)
      REAL    ymax(nyres), ymin(nyres), ywid(nyres)
      common /cdmesh/  idir, jdir, xmax, xmin, xwid, ymax, ymin, ywid

      REAL   xh(im), yh(jm), xz(im), yz(jm)
      REAL   dxh(im), dyh(jm), dxz(im), dyz(jm)
      common /coord2/ xh, yh, xz, yz, dxh, dyh, dxz, dyz

      integer i, j, l, m, n
      REAL    dx(imt), dy(jmt)
      REAL    sum

      m = 1
      do l = 1,nxres
         call cmesh (dx(m), xmax(l), xmin(l), xwid(l), n, idir(l))
         m = m+n
      end do
      m = m-1
      sum = 0.
      do i = 1,m
         sum = sum+dx(i)
      end do
      write (7,1)
      write (7,90) dx
      write (7,89) sum
      m = 1
      do l = 1,nyres
         call cmesh (dy(m), ymax(l), ymin(l), ywid(l), n, jdir(l))
         m = m+n
      end do
      m = m-1
      sum = 0.
      do j = 1,m
         sum = sum+dy(j)
      end do
      write (7,2)
      write (7,90) dy
      write (7,89) sum

    1 format (/,10x,'dxt=')
    2 format (/,10x,'dyt=')
   89 format (1x,e14.7)
   90 format (1x,8e10.5)
      end


C******************************************************************
C******************************************************************
      subroutine grids
      implicit none

      integer    im   , jm   , mx      , nxres  , nyres
      parameter (im=18, jm=30, mx=im*jm, nxres=1, nyres=1)
      integer    imt   , jmt   , km  , itte
      parameter (imt=im, jmt=jm, km=1, itte=100)

      REAL     boxtw, boxts, xt(imt), yt(jmt), xv(imt), yv(jmt)
      REAL     deltaz(km)
      common /coord/ boxtw, boxts, xt, yt, xv, yv, deltaz

      integer idir(nxres), jdir(nyres)
      REAL    xmax(nxres), xmin(nxres), xwid(nxres)
      REAL    ymax(nyres), ymin(nyres), ywid(nyres)
      common /cdmesh/  idir, jdir, xmax, xmin, xwid, ymax, ymin, ywid

      REAL   xh(im), yh(jm), xz(im), yz(jm)
      REAL   dxh(im), dyh(jm), dxz(im), dyz(jm)
      common /coord2/ xh, yh, xz, yz, dxh, dyh, dxz, dyz

      REAL    cf(jm), c3(jm), c4(jm) , c5(jm), c5a(im), c6(jm)
      REAL    c7(jm), c8(jm), c8a(im), c9(jm)
      common /coef/ cf, c3, c4, c5, c5a, c6, c7, c8, c8a, c9

      REAL    be(im,jm), bu(im,jm), bv(im,jm)
      common /masks/ be, bu, bv

      REAL    pi, dt2, degrad, epsln, dt, secpday, one, zero 
      REAL    eighth, quarter, half, two, four, depthr, omega, days
      REAL    avgper
      integer ittend
      common /rscala1/ pi, dt2, degrad, epsln, dt, secpday
     1               , one, zero, eighth, quarter, half, two, four
     2               , depthr, omega, days, avgper, ittend

      REAL    grav, depth, rober
      REAL    eanom, escale, txanom, txscale
      common /rscala2/ grav, depth, rober
     1               , eanom, escale, txanom, txscale

      REAL    h(im,jm), uh(im,jm), vh(im,jm), z(im,jm)
      common /var4/  h, uh, vh, z

      REAL    ud(im,jm,0:itte),vd(im,jm,0:itte),ed(im,jm,0:itte)      
      common /modoup/ ud, vd, ed

      integer i, j
      REAL    degcm

      call mesh (dxh, dyh)

      xh(1)  = boxtw+.5*dxh(1)
      yh(1)  = boxts+.5*dyh(1)
      xz(1)  = boxtw
      yz(1)  = boxts
      dxz(1) = dxh(1)
      dyz(1) = dyh(1)
      do i = 2,im
         dxz(i)=.5*(dxh(i)+dxh(i-1))
         xh(i)=xh(i-1)+dxz(i)
         xz(i)=xz(i-1)+dxh(i-1)
      end do
      do j = 2,jm
         dyz(j)=.5*(dyh(j)+dyh(j-1))
         yh(j)=yh(j-1)+dyz(j)
         yz(j)=yz(j-1)+dyh(j-1)
      end do
      write (7,301) xh
      write (7,302) yh
      degcm=111.324e5
      do i = 1,im
         dxh(i)=dxh(i)*degcm
         dxz(i)=dxz(i)*degcm
      end do
      do j = 1,jm
         dyh(j)=dyh(j)*degcm
         dyz(j)=dyz(j)*degcm
      end do

 301  format (/,/,/,' xh(i),i=1,im',(/,1x,10f7.3))
 302  format (/,/,/,' yh(j),j=1,jm',(/,1x,10f7.3))

      end


C******************************************************************
C******************************************************************
      subroutine initocn
      implicit none

      integer    im   , jm   , mx      , nxres  , nyres
      parameter (im=18, jm=30, mx=im*jm, nxres=1, nyres=1)
      integer    imt   , jmt   , km  , itte
      parameter (imt=im, jmt=jm, km=1, itte=100)

      REAL     boxtw, boxts, xt(imt), yt(jmt), xv(imt), yv(jmt)
      REAL     deltaz(km)
      common /coord/ boxtw, boxts, xt, yt, xv, yv, deltaz

      integer idir(nxres), jdir(nyres)
      REAL    xmax(nxres), xmin(nxres), xwid(nxres)
      REAL    ymax(nyres), ymin(nyres), ywid(nyres)
      common /cdmesh/  idir, jdir, xmax, xmin, xwid, ymax, ymin, ywid

      REAL   xh(im), yh(jm), xz(im), yz(jm)
      REAL   dxh(im), dyh(jm), dxz(im), dyz(jm)
      common /coord2/ xh, yh, xz, yz, dxh, dyh, dxz, dyz

      REAL    cf(jm), c3(jm), c4(jm) , c5(jm), c5a(im), c6(jm)
      REAL    c7(jm), c8(jm), c8a(im), c9(jm)
      common /coef/ cf, c3, c4, c5, c5a, c6, c7, c8, c8a, c9

      REAL    be(im,jm), bu(im,jm), bv(im,jm)
      common /masks/ be, bu, bv

      REAL    pi, dt2, degrad, epsln, dt, secpday, one, zero 
      REAL    eighth, quarter, half, two, four, depthr, omega, days
      REAL    avgper
      integer ittend
      common /rscala1/ pi, dt2, degrad, epsln, dt, secpday
     1               , one, zero, eighth, quarter, half, two, four
     2               , depthr, omega, days, avgper, ittend

      REAL    grav, depth, rober
      REAL    eanom, escale, txanom, txscale
      common /rscala2/ grav, depth, rober
     1               , eanom, escale, txanom, txscale

      REAL    h(im,jm), uh(im,jm), vh(im,jm), z(im,jm)
      common /var4/  h, uh, vh, z

      REAL    ud(im,jm,0:itte), vd(im,jm,0:itte), ed(im,jm,0:itte)    
      common /modoup/ ud, vd, ed
cc---
      data boxtw, boxts /0.0,-15.0 /
      data xmax/1./
      data xmin/1./
      data xwid/18./
      data idir/1/
      data ymax/1./
      data ymin/1./
      data ywid/30./
      data jdir/1/

      integer nr, nw, nuread, itt

      data  epsln,  dt,  nr, nw, nuread, grav,  itt,  days
     1    /1.e-25,  0.0, 51, 50,   55,   2.,    1,   1.0/
      data avgper, rober,   one,  zero,  secpday
     1  /   1.0,   0.01,    1.0,   0.0,  86400.0/
      data  eighth, quarter,  half,   two,   four/
     1      0.125,   0.25,     0.5,   2.0,   4.0/
      data depth,eanom,escale,txanom,txscale/9800.,2.0,5.0,0.0,20./

      integer i, j, im1, jm1, k
      integer ntspd, ittend1
      REAL    hmax, dxmin, dymin, cormax, cyh, dsq, cfl
      REAL    cor, percnt

      pi      = 3.1415926535
      degrad  = pi/180.
      omega   = pi/43082.0
      do i = 1,im
         c5a(i)  = zero
         c8a(i)  = zero
      end do

      do j = 1,jm
         cf(j)  = zero
         c3(j)  = zero
         c4(j)  = zero
         c5(j)  = zero
         c6(j)  = zero
         c7(j)  = zero
         c8(j)  = zero
         c9(j)  = zero
      end do

      do j = 1,jm
         do i = 1,im
            h(i,j)   = zero
            z(i,j)   = zero
            uh(i,j)  = zero
            vh(i,j)  = zero
         end do
      end do

      call grids

      do j = 1,jm
         do i = 1,im
            h(i,j) =depth
            if((j.eq.1).or.(j.eq.jm).or.(i.eq.1).or.(i.eq.im))
     &           h(i,j) = zero
         end do
      end do

      depthr = one/depth

      do j = 1,jm
         jm1 = j-1
         if (j .eq. 1) jm1 = j
         do i = 1,im
            im1 = i-1
            if (i .eq. 1) im1 = i
            be(i,j)=zero
            if (h(i,j) .gt. epsln)     be(i,j) = one
            bu(i,j) = zero
            bv(i,j) = zero
            if ((h(i,j).eq.depth).and.(h(im1,j).eq.depth)) then
               bu(i,j) = one
            endif
            if ((h(i,j).eq.depth).and.(h(i,jm1).eq.depth)) then
               bv(i,j) = one
            endif
         end do
      end do
      do j = 2,jm
         do i = 2,im
            if ((bu(i,j).eq.one).or.(bu(i,j-1).eq.one).or.
     &           (bv(i-1,j).eq.one).or.(bv(i,j).eq.one)) then
               z(i,j) = cf(j)*depthr*
     &              (h(i,j)+h(i-1,j)+h(i,j-1)+h(i-1,j-1))/
     &              ((h(i,j) + h(i-1,j))*c7(j) + 
     &              (h(i,j-1) + h(i-1,j-1))* c7(j-1) + epsln)
            endif
         end do
      end do
      do i = 1,im
         c5a(i) = one/dxz(i)
         c8a(i) = one/dxh(i)
      end do
      do j = 1,jm
         c3(j) = cos(yz(j)*degrad)
         c4(j) = eighth/cos(yh(j)*degrad)
         c5(j) = one/cos(yh(j)*degrad)
         c6(j) = one/dyz(j)
         c7(j) = cos(yh(j)*degrad)
         c8(j) = one/cos(yh(j)*degrad)
         c9(j) = one/(cos(yh(j)*degrad)*dyh(j))
         cf(j) = two*omega*sin(yz(j)*degrad)*c3(j)
      end do
      hmax   = zero
      dxmin  = dxh(1)*cos(yh(1)*degrad)
      dymin  = dyh(1)
      cormax = abs(two*omega*sin(yz(1)*degrad))
      do j = 1,jm
         dymin  = min(dyh(j),dymin)
         cormax = max(cormax,abs(two*omega*sin(yz(j)*degrad)))
         cyh    = cos(yh(j)*degrad)
         do i = 1,im
            dxmin = min(dxh(i)*cyh,dxmin)
            hmax  = max(h(i,j),hmax)
         end do
      end do
      dsq    = one/dxmin**2 + one/dymin**2
      cfl    = one/(two*sqrt(grav*hmax*dsq))
      cor    = one/cormax
      percnt = 0.9
      dt     = percnt*min(cfl,cor)
      write (7,'(6(a7,e9.4))') '    dt=',dt, '   cfl=', cfl,
     1                         '   cor=', cor, ' dxmin=', dxmin,
     2                         ' dymin=', dymin, '  hmax=', hmax
      ntspd  = secpday/dt
      dt     = secpday/(ntspd+1)
      dt2    = two*dt
      ittend = nint(itt + (ntspd+1)*days)
      ittend1=ittend
      ittend = 30
      write (7,'(1x,a15,i4,a17,e14.7,a4)') 'md use ', ntspd+1,
     1     ' ts/day using dt=',dt,' sec'

CC-----read file
c      open(10,file='ELEV')
c      open(11,file='UVELO')
c      open(12,file='VVELO')

c      read(10,*)(((ed(i,j,ik),i=1,im),j=1,jm),ik=1,ittend1+2)
c      read(11,*)(((ud(i,j,ik),i=1,im),j=1,jm),ik=1,ittend1+2)
c      read(12,*)(((vd(i,j,ik),i=1,im),j=1,jm),ik=1,ittend1+2)

c      rewind (10)
c      rewind (11)
c      rewind (12)

      do k = 1,ittend1+2
         do j = 1,jm
            do i = 1,im
               ed(i,j,k) = 0.0
               ud(i,j,k) = 0.0
               vd(i,j,k) = 0.0
            enddo
         enddo
      enddo
     
      end


C******************************************************************
C******************************************************************
      subroutine numbmod( n )
      implicit none

      integer n

      integer    im   , jm   , mx      , nxres  , nyres
      parameter (im=18, jm=30, mx=im*jm, nxres=1, nyres=1)
      integer    imt   , jmt   , km  , itte
      parameter (imt=im, jmt=jm, km=1, itte=100)

      integer nu, nv, ne

      nu = (im-1)*(jm-1)
      nv = (im-1)*(jm-1)
      ne = (im-1)*(jm-1)

      n  = ne+nu+nv + 1

      end



C******************************************************************
C******************************************************************
      subroutine initmod(nn,x)
      implicit none

      integer nn
      REAL    x(nn)

      integer    im   , jm   , mx      , nxres  , nyres
      parameter (im=18, jm=30, mx=im*jm, nxres=1, nyres=1)
      integer    imt   , jmt   , km  , itte
      parameter (imt=im, jmt=jm, km=1, itte=100)

      REAL    ud(im,jm,0:itte), vd(im,jm,0:itte), ed(im,jm,0:itte)    
      common /modoup/ ud, vd, ed

      REAL    pi, dt2, degrad, epsln, dt, secpday, one, zero 
      REAL    eighth, quarter, half, two, four, depthr, omega, days
      REAL    avgper
      integer ittend
      common /rscala1/ pi, dt2, degrad, epsln, dt, secpday
     1               , one, zero, eighth, quarter, half, two, four
     2               , depthr, omega, days, avgper, ittend

      common /loop/nu,nv,ne,loc

      integer nu,nv,ne,loc
      integer i
      REAL    tfr, tfn, fr, fn

      call initocn

      loc = 0
      nu  = (im-1)*(jm-1)
      nv  = (im-1)*(jm-1)
      ne  = (im-1)*(jm-1)
      loc = ne+nu+nv

      do i = 1,loc
         x(i) = 0.01
      enddo
      tfr = 120.
      tfn = 9999.
      fr  = one/(tfr*secpday)
      fn  = one/(tfn*secpday)
      x(loc+1) = fr

      end


C******************************************************************
C******************************************************************
      SUBROUTINE POSTMOD( NN, XX, FC )
      implicit none

C================================================
C The subroutine "POSTMOD" is called after
C the optimization.
C It should contain the output of the results.
C================================================
      integer nn
      REAL    xx(nn), fc

      integer    im   , jm   , mx      , nxres  , nyres
      parameter (im=18, jm=30, mx=im*jm, nxres=1, nyres=1)
      integer    imt   , jmt   , km  , itte
      parameter (imt=im, jmt=jm, km=1, itte=100)

      REAL    e(im,jm), u(im,jm), v(im,jm)

      integer       nu, nv, ne, loc
      common /loop/ nu, nv, ne, loc

      integer i, j, np, fr

      PRINT *,'**************************************'
      PRINT *,'***     RESULT OF OPTIMISATION      **'
      PRINT *,'**************************************'
      PRINT 9010, FC
      PRINT 9020

      do j = 1,jm-1
         np=(j-1)*(im-1)
         do i = 1,im-1
            e(i,j) = xx(np+i)
         end do
      end do

      do j=1,jm-1
         np=(j-1)*(im-1)
         do i = 2,im
            u(i,j) = xx(np+i+ne)
         end do
      end do

      do j = 2,jm
         np=(j-2)*(im-1)
         do i = 1,im-1
            v(i,j) = xx(np+i+ne+nu)
         end do
      end do

      open(15,file='final_result')
      write(15,*)' e opimization'
      write(15,'(17f7.4)')((e(i,j),i=1,17),j=1,jm-1)
      write(15,*)' u opimization'
      write(15,'(17f7.4)')((u(i,j),i=2,18),j=1,jm-1)
      write(15,*)' v opimization'
      write(15,'(17f7.4)')((v(i,j),i=1,17),j=2,jm)
      write(15,*)' fr opimization'
      fr=xx(nn)    
      write(15,*)'fr====',fr

 9010 FORMAT(1X,'The value of the costfunction is : ',E12.6)
 9020 FORMAT(1X,'The optimal parameter values are :')
 9030 FORMAT(1X,I5,2X,E12.6)

      END


C******************************************************************
C******************************************************************
      subroutine step( ddt, fr
     $               , uold, vold, eold
     $               , uact, vact, eact
     $               , unew, vnew, enew
     $               )
      implicit none

      integer    im   , jm   , mx      , nxres  , nyres
      parameter (im=18, jm=30, mx=im*jm, nxres=1, nyres=1)
      integer    imt   , jmt   , km  , itte
      parameter (imt=im, jmt=jm, km=1, itte=100)

      REAL     ddt, fr
      REAL     uold(im,jm), vold(im,jm), eold(im,jm)
      REAL     uact(im,jm), vact(im,jm), eact(im,jm)
      REAL     unew(im,jm), vnew(im,jm), enew(im,jm)

      REAL    cf(jm), c3(jm), c4(jm) , c5(jm), c5a(im), c6(jm)
      REAL    c7(jm), c8(jm), c8a(im), c9(jm)
      common /coef/ cf, c3, c4, c5, c5a, c6, c7, c8, c8a, c9

      REAL    be(im,jm), bu(im,jm), bv(im,jm)
      common /masks/ be, bu, bv
     
      REAL    pi, dt2, degrad, epsln, dt, secpday, one, zero 
      REAL    eighth, quarter, half, two, four, depthr, omega, days
      REAL    avgper
      integer ittend
      common /rscala1/ pi, dt2, degrad, epsln, dt, secpday
     1               , one, zero, eighth, quarter, half, two, four
     2               , depthr, omega, days, avgper, ittend

      REAL    grav, depth, rober
      REAL    eanom, escale, txanom, txscale
      common /rscala2/ grav, depth, rober
     1               , eanom, escale, txanom, txscale

      REAL    h(im,jm), uh(im,jm), vh(im,jm), z(im,jm)
      common /var4/  h, uh, vh, z

      integer i, j

      do j = 1,jm
         do i = 2,im
            uh(i,j) = half*(h(i,j)+h(i-1,j))*uact(i,j)*bu(i,j)
         end do
      end do

      do j = 2,jm
         do i = 1,im
            vh(i,j) = half*(h(i,j)+h(i,j-1))*vact(i,j)*bv(i,j)
         end do
      end do

      do j = 1,jm-1
         do i = 2,im
            unew(i,j) = uold(i,j) + ddt*bu(i,j)*
     &           ( c4(j)*(z(i,j)+z(i,j+1))*
     &           ((vh(i,j)+vh(i-1,j))*c3(j) +
     &           (vh(i,j+1)+vh(i-1,j+1))*c3(j+1))
     &           - c5(j)*c5a(i)*grav*(eact(i,j)-eact(i-1,j))
     &           - fr*uold(i,j)
     &           )
         end do
      end do

      do j = 2,jm
         do i = 1,im-1
            vnew(i,j) = vold(i,j) + ddt*bv(i,j)*
     &           ( - eighth*(z(i,j)+z(i+1,j))*
     &           (uh(i,j) + uh(i+1,j) + uh(i,j-1) + uh(i+1,j-1))
     &           - c6(j)*grav*(eact(i,j)-eact(i,j-1))
     &           - fr*vold(i,j)
     &           )
         end do
      end do

      do j = 1,jm-1
         do i = 1,im-1
            enew(i,j) = eold(i,j) + ddt*depthr*h(i,j)*
     &           ( -c8(j)*c8a(i)*(uh(i+1,j)-uh(i,j))
     &           -c9(j)*(vh(i,j+1)*c3(j+1)-vh(i,j)*c3(j))
     &           )
         end do
      end do

      end


C******************************************************************
      subroutine cost( itt, umod, vmod, emod, fc )
      implicit none

      integer    im   , jm   , mx      , nxres  , nyres
      parameter (im=18, jm=30, mx=im*jm, nxres=1, nyres=1)
      integer    imt   , jmt   , km  , itte
      parameter (imt=im, jmt=jm, km=1, itte=100)

      integer itt
      REAL    umod(im,jm), vmod(im,jm), emod(im,jm), fc

      REAL    pi, dt2, degrad, epsln, dt, secpday, one, zero 
      REAL    eighth, quarter, half, two, four, depthr, omega, days
      REAL    avgper
      integer ittend
      common /rscala1/ pi, dt2, degrad, epsln, dt, secpday
     1               , one, zero, eighth, quarter, half, two, four
     2               , depthr, omega, days, avgper, ittend

      REAL    ud(im,jm,0:itte), vd(im,jm,0:itte), ed(im,jm,0:itte)    
      common /modoup/ ud, vd, ed

      integer i, j

      fc = 0.0
      do i = 1,im
         do j = 1,jm
            fc = fc + (umod(i,j)-ud(i,j,itt))**2
     &              + (vmod(i,j)-vd(i,j,itt))**2
     &              + (emod(i,j)-ed(i,j,itt))**2
         end do
      end do

      end


C******************************************************************
      subroutine model(nn,x,fc)
      implicit none

      integer nn
      REAL    x(nn), fc

      integer    im   , jm   , mx      , nxres  , nyres
      parameter (im=18, jm=30, mx=im*jm, nxres=1, nyres=1)
      integer    imt   , jmt   , km  , itte
      parameter (imt=im, jmt=jm, km=1, itte=100)

      REAL    cf(jm), c3(jm), c4(jm) , c5(jm), c5a(im), c6(jm)
      REAL    c7(jm), c8(jm), c8a(im), c9(jm)
      common /coef/ cf, c3, c4, c5, c5a, c6, c7, c8, c8a, c9

      REAL    be(im,jm), bu(im,jm), bv(im,jm)
      common /masks/ be, bu, bv
     
      REAL    pi, dt2, degrad, epsln, dt, secpday, one, zero 
      REAL    eighth, quarter, half, two, four, depthr, omega, days
      REAL    avgper
      integer ittend
      common /rscala1/ pi, dt2, degrad, epsln, dt, secpday
     1               , one, zero, eighth, quarter, half, two, four
     2               , depthr, omega, days, avgper, ittend

      REAL    grav, depth, rober
      REAL    eanom, escale, txanom, txscale
      common /rscala2/ grav, depth, rober
     1               , eanom, escale, txanom, txscale

      REAL    h(im,jm), uh(im,jm), vh(im,jm), z(im,jm)
      common /var4/  h, uh, vh, z

      common /lo1/mp      

      integer mp,mp1,mp2
      integer i, j, itt, iout
      integer loc, loce, loceu
      REAL    fr, fn
      REAL    fc1

      REAL    u1(im,jm), u2(im,jm), u3(im,jm)
      REAL    v1(im,jm), v2(im,jm), v3(im,jm)
      REAL    e1(im,jm), e2(im,jm), e3(im,jm)

      integer    nouter
      parameter( nouter = 3 )

CADJ INIT ftape1  = common, 1
CADJ INIT fouter  = common, nouter

      fr = 0.0
      fn = 0.0
      fc = 0.0

CC=========================
CC set up initials
CC=========================
      do j = 1,jm
         do i = 1,im
            u1(i,j) = zero
            v1(i,j) = zero
            e1(i,j) = zero
            u2(i,j) = zero
            v2(i,j) = zero
            e2(i,j) = zero
            u3(i,j) = zero
            v3(i,j) = zero
            e3(i,j) = zero
         end do
      end do

      loc=0
      mp=0
      do j = 1,jm-1
         mp=(j-1)*(im-1)
         do i = 1,im-1
            loc=loc+1
            e1(i,j) = x(mp+i) * 1.e+4
         end do
      end do

      loce=loc        
      do j = 1,jm-1
         mp1=(j-1)*(im-1)
         do i = 2,im
            loc=loc+1
            u1(i,j) = x(mp1+i-1+loce)
         end do
      end do

      loceu=loc
      do j = 2,jm
         mp2=(j-2)*(im-1)
         do i = 1,im-1
            loc=loc+1
            v1(i,j) = x(mp2+i+loceu)
         end do
      end do

      fr = x(loc+1)/ 1.e+6

CADJ  store u1 = ftape1
CADJ  store v1 = ftape1
CADJ  store e1 = ftape1

CC================================================
CC costfunction term for first time level
CC================================================
      call cost( 2, u1, v1, e1, fc1 )
      fc = fc+fc1

CC================================================
CC    Eular forward differential (one time step)
CC    which is actually setting initials, also for 
CC    leap-frog differentials 
CC================================================
      call step( dt, fr, u1, v1, e1, u1, v1, e1, u2, v2, e2 )

CADJ  store u2 = ftape1
CADJ  store v2 = ftape1
CADJ  store e2 = ftape1

CC================================================
CC costfunction term for second time level
CC================================================
      call cost( 2, u2, v2, e2, fc1 )
      fc = fc+fc1

CC======================================================
CC leap-frog integration (two time step with nm,n===>np
CC======================================================
      do iout = 1, nouter

CADJ store u1,v1,e1,u2,v2,e2 = fouter

CADJ init  ftape = common, 10

      do itt = 1, ittend, 3

CADJ  store u1 = ftape
CADJ  store v1 = ftape

CADJ  store u2 = ftape
CADJ  store v2 = ftape

          call step( dt2, fr, u1, v1, e1, u2, v2, e2, u3, v3, e3 )
CADJ  store u3 = ftape
CADJ  store v3 = ftape
CADJ  store e3 = ftape
          call cost( itt, u3, v3, e3, fc1 )

          call step( dt2, fr, u2, v2, e2, u3, v3, e3, u1, v1, e1 )
CADJ  store u1 = ftape
CADJ  store v1 = ftape
CADJ  store e1 = ftape
          call cost( itt, u1, v1, e1, fc1 )

          call step( dt2, fr, u3, v3, e3, u1, v1, e1, u2, v2, e2 )
CADJ  store u2 = ftape
CADJ  store v2 = ftape
CADJ  store e2 = ftape
          call cost( itt, u2, v2, e2, fc1 )

        fc = fc+fc1
      end do
      end do

      end

