C ========================================================================
      subroutine costcal(n)
      implicit none
      integer    km   , irechis         , irecobs
      parameter (km=10, irechis=8*(km+1), irecobs=8*2*km)   

      REAL         f    , fn    , s    , w, cost
      common/vars/ f(km), fn(km), s(km), w, cost

      REAL         delt, v    , fo    , a
      integer                                  nm, mkobs
      common/cons/ delt, v(km), fo(km), a(km), nm, mkobs

      integer n
      integer k, j
CC      print *,'enter costcal n=', n
CC      do 10 k = 1, km
CC         fo(k) = f(k)
CC         v(k)  = 1.0
CC 10   continue
CC      write(12,rec=n) (fo(k),k=1,km),(v(k),j=1,km)
CC      print *, n

      read(12,rec=n) 
     1    (fo(k),k=1,km),(v(j),j=1,km)       ! Read in obs fo and weights v

      do 100 k=1,km                          ! for time n-1
        cost = cost+0.5*v(k)*(f(k)-0.9*fo(k))**2
 100  continue
CC      print *, n
      end

C ========================================================================
      subroutine wcal
      implicit none
      integer    km   , irechis         , irecobs
      parameter (km=10, irechis=8*(km+1), irecobs=8*2*km)   

      REAL         f    , fn    , s    , w, cost
      common/vars/ f(km), fn(km), s(km), w, cost

      REAL         delt, v    , fo    , a
      integer                                  nm, mkobs
      common/cons/ delt, v(km), fo(km), a(km), nm, mkobs

      integer k

      w = 0.0
      do 100 k=1,km
        w = w+a(k)*f(k)
 100  continue
      end

C =======================================================================
      subroutine scal
      implicit none
      integer    km   , irechis         , irecobs
      parameter (km=10, irechis=8*(km+1), irecobs=8*2*km)   

      REAL         f    , fn    , s    , w, cost
      common/vars/ f(km), fn(km), s(km), w, cost

      REAL         delt, v    , fo    , a
      integer                                  nm, mkobs
      common/cons/ delt, v(km), fo(km), a(km), nm, mkobs

      integer k

      do 100 k=1,km
        s(k) = f(k)*w
 100  continue
      end

C =======================================================================
      subroutine fcal             ! Steps model forward to time n
      implicit none
      integer    km   , irechis         , irecobs
      parameter (km=10, irechis=8*(km+1), irecobs=8*2*km)   

      REAL         f    , fn    , s    , w, cost
      common/vars/ f(km), fn(km), s(km), w, cost

      REAL         delt, v    , fo    , a
      integer                                  nm, mkobs
      common/cons/ delt, v(km), fo(km), a(km), nm, mkobs

      integer k

      do 100 k=1,km
        fn(k) = f(k)-delt*s(k)
 100  continue
      do 200 k=1,km               ! Shift fn into f for next time loop
        f(k) = fn(k)
 200  continue
      end

C ======================================================================
      subroutine model( nn, xx, fc )
      implicit none
      integer nn
      REAL    xx(nn), fc

C   Field f is 2-d, e.g. frequency (k) and time (n).  Model implemented
C   is
C                 f(k,n)   = f(k,n-1) - s(k,n-1)*delt,
C                 s(k,n-1) = f(k,n-1)*w(n-1), 
C                              km
C                 w(n-1)   =  SUM [a(k)*f(k,n-1)],
C                             k=1
C                          nm   km
C                 cost =  SUM {SUM [v(k,n)*(f(k,n)-fo(k,n)**2]}
C                         n=0  k=1
C
C   a(k) is a set of weights defining the integral property w(n); a(k)
C   and the initial conditions f(k,0) comprise the control variables.  
C   v(k,n) is a set of weights defining the cost via the integral shown
C   and is taken as constant. v(k,n)=0 wherever there are no observations 
C   fo(k,n).
C
      integer    km   , irechis         , irecobs
      parameter (km=10, irechis=8*(km+1), irecobs=8*2*km)   

      REAL         f    , fn    , s    , w, cost
      common/vars/ f(km), fn(km), s(km), w, cost

      REAL         delt, v    , fo    , a
      integer                                  nm, mkobs
      common/cons/ delt, v(km), fo(km), a(km), nm, mkobs

      integer k, n

C-------------------------------
C define file tape
C-------------------------------
CADJ INIT mytape = 'tape'


C   Begin main model loop:

      cost = 0.0                 ! Initialize cost accumulator
      do 50 k=1,nn               ! Copy np control vars into model
	f(k) = xx(k)		 
 50   continue 

C     --------------------------------------------------------------------+
      do 100 n=1,nm              ! Start loop over times n=1,nm
CADJ    STORE f = mytape	 ! TAMC saves f to a file
        call costcal(n)          ! Accumulate cost from obs at time n-1
        call wcal                ! Compute w(n-1) from f(k,n-1), a(k)
CADJ    STORE w = mytape	 ! TAMC saves w to a file
        call scal                ! Compute s(k,n-1) from w(n-1), f(k,n-1)
        call fcal                ! Step f(k,n-1) forward to f(k,n)=fn(k)
 100  continue                   ! End of time-stepping loop

C     --------------------------------------------------------------------+
CADJ  STORE f = mytape		 ! TAMC saves f to a file
      call costcal(nm+1)         ! Accumulate cost from obs at time nm
      fc = cost			 ! write final result to FC

C     --------------------------------------------------------------------+
      end 
C =======================================================================
      subroutine inp
C                              ! Reads in initial values of delt, f and a)
      integer    km   , irechis         , irecobs
      parameter (km=10, irechis=8*(km+1), irecobs=8*2*km)   

      REAL         f    , fn    , s    , w, cost
      common/vars/ f(km), fn(km), s(km), w, cost

      REAL         delt, v    , fo    , a
      integer                                  nm, mkobs
      common/cons/ delt, v(km), fo(km), a(km), nm, mkobs


      write(6,*)
c     write(6,*) ' s/r in called:'
      read(10,909) delt, nm, mkobs
 909  format(f8.0,2i8)
        write(6,*) ' delt=',delt,'  nm=',nm,'  mkobs=',mkobs
      read(10,910)(f(k),k=1,km),(a(j),j=1,km)
 910  format(10f8.0)
        write(6,*)' Initial f(k) and a(k):'
        write(6,911)(f(k),k=1,km),(a(j),j=1,km)
 911    format(1x,10f7.3)
        write(6,*) ' '

      end

c ======================================================================
      subroutine initmod(np, f0)
      implicit none
      integer    km   , irechis         , irecobs
      parameter (km=10, irechis=8*(km+1), irecobs=8*2*km)   

      REAL         f    , fn    , s    , w, cost
      common/vars/ f(km), fn(km), s(km), w, cost

      REAL         delt, v    , fo    , a
      integer                                  nm, mkobs
      common/cons/ delt, v(km), fo(km), a(km), nm, mkobs


      integer np
      REAL  f0(km)

      integer k

      open(unit=10,file='controls.dem',status='OLD',
     1         access='SEQUENTIAL')
c     open(unit=11,file='history.dem',access='DIRECT',recl=rechis,
c    1         status='UNKNOWN')
      open(unit=12,file='obs.dem',access='DIRECT',recl=irecobs,
     1          status='UNKNOWN')

      call inp                      ! Read in delt,nm,mkobs,f(k) at n=0 & a(k)
      do 50 k=1,km                  ! Save model ICs in f0(k)
         f0(k)=f(k)
 50   continue

      end


c ====================================================================
      subroutine numbmod( n )

      integer    km   , irechis         , irecobs
      parameter (km=10, irechis=8*(km+1), irecobs=8*2*km)   

      REAL         f    , fn    , s    , w, cost
      common/vars/ f(km), fn(km), s(km), w, cost

      REAL         delt, v    , fo    , a
      integer                                  nm, mkobs
      common/cons/ delt, v(km), fo(km), a(km), nm, mkobs

      integer n
      n = km
      return 
      end


c ====================================================================
      subroutine postmod(np, f0, fc, fgrad)
      implicit none

      integer    km   , irechis         , irecobs
      parameter (km=10, irechis=8*(km+1), irecobs=8*2*km)   

      REAL         f    , fn    , s    , w, cost
      common/vars/ f(km), fn(km), s(km), w, cost

      REAL         delt, v    , fo    , a
      integer                                  nm, mkobs
      common/cons/ delt, v(km), fo(km), a(km), nm, mkobs

      integer np
      REAL    f0(km)
      REAL    fc
      REAL    fgrad(km)

      integer i

      write(6,*)
      write(6,*)' np=',np,'  cost=',fc,'   F0 follows:'
      write(6,990) (f0(i),i=1,km)
      write(6,*) ' '
      write(6,*) ' Gradient FGRAD follows:'
      write(6,990) (fgrad(i),i=1,km)
 990  format(1x,10f7.3)

c      FC0=FC
c      DELTA = 0.001
c      do 100 i=1,km
c	 f0(i)=f0(i)+DELTA
c         call model( Np, F0, FC )
c         FGRAD(i)=(FC-FC0)/DELTA
c         f0(i)=f0(i)-DELTA
c 100  continue
c      write(6,990) (fgrad(i),i=1,km)

      close(10)
      close(11)

      return
      end



      






