
#include "define.h"
#include "tsearch.h"

#ifdef CRAY

#define ADWRITE ADWRITE
#define ADREAD  ADREAD
#define ADSTORE ADSTORE
#define ADRESTO ADRESTO

#else

#ifdef AIX

#define ADWRITE adwrite
#define ADREAD  adread
#define ADSTORE adstore
#define ADRESTO adresto

#else

#define ADWRITE adwrite_
#define ADREAD  adread_
#define ADSTORE adstore_
#define ADRESTO adresto_

#endif
#endif


#define MAXTAPE 20

typedef  int    (*Comp)   PP((Const Void *, Const Void *));
typedef  Void   (*Action) PP((Const Void *, VISIT, int));


/***********************************************************/
struct node { 
  FORINT key;
  union{ long  irec; char *memo; } UU;
};

struct node node_space;
struct node *nodeptr = &node_space;


/***********************************************************/
struct code {
  char        *name;
  FILE        *stream;
  struct node *root;
};

struct code code_space;
struct code *codeptr = &code_space;
struct code *code_root = NULL;


/***********************************************************/
struct memo {
  char        *name;
  struct node *root;
};

struct memo memo_space;
struct memo *memoptr = &memo_space;
struct memo *memo_root = NULL;


/***********************************************************/
/*          NODE_COMPARE                                   */
/***********************************************************/
Inline
Static int
node_compare(node1, node2)
     struct node *node1, *node2;
{
  int i;

  if (node1->key > node2->key) {
    return 1;
  }
  else if (node1->key < node2->key) {
    return -1;
  }
  else
    return 0;
}


/***********************************************************/
/*          PRINT_NODE                                   */
/***********************************************************/
Inline
Void
print_node(node, order, level)
     Const struct node **node;
     VISIT order;
     int level;
{
  struct node *p;

  p = (struct node *) *node;
  if (order == postorder || order == leaf) {
    printf( "keys = %i, rec = %l\n", p->key, p->UU.irec);
  }
}



/***********************************************************/
/*          CODE_COMPARE                                   */
/***********************************************************/
Inline
Static int
code_compare(node1, node2)
     struct code *node1, *node2;
{
  return strcmp( node1->name, node2->name );
}

/***********************************************************/
/*          GETCODE                                        */
/***********************************************************/
Inline
Static struct code *
getcode( s )
     char *s;
{
  struct code **found;

  codeptr->name = s;
  found = (struct code **)
    tsearch( (Anyptr) codeptr, (Anyptr *) &code_root, (Comp) code_compare );
  if (*found == codeptr) {

    (*found)->root   = NULL;
    (*found)->stream = fopen( s, "w+" );

    if (((*found)->name = (char *) malloc( (size_t) strlen(s)+1 )) == NULL)
      fprintf( stderr, "getcode : malloc failed\n" );
    strcpy((*found)->name,s);

    if ((codeptr = (struct code *) malloc( sizeof(struct code) )) == NULL)
      fprintf( stderr, "getcode : malloc failed\n" );
  }
  return (*found);
}


/***********************************************************/
/*          ADWRITE                                        */
/***********************************************************/
Void
ADWRITE( tfile, len, ptr, size, items, ikey )
     char    tfile[];
     FORINT  *len;
     Void    *ptr;
     FORINT  *size;
     FORINT  *items;
     FORINT  *ikey;
{
  int         ierr;
  struct code *cptr;
  struct node **found;
  int         i;
  char        tape[MAXTAPE];

  for( i=0; i<*len; i++) tape[i] = tfile[i]; tape[*len] = '\0';
  cptr = getcode( (char*) tape);

  nodeptr->key = *ikey;

  found = (struct node **)
    tsearch((Anyptr) nodeptr, (Anyptr *) &(cptr->root), (Comp) node_compare );

  if (*found == nodeptr) {    /* node was inserted, so get a new one */
    if ((nodeptr = (struct node *) malloc( sizeof(struct node) )) == NULL)
      fprintf( stderr, "ADWRITE : malloc failed\n" );

    if ((ierr = fseek( cptr->stream, 0L, 2 )) != 0) {
      fprintf( stderr, "ADWRITE : error in fseek = %i\n", ierr );
      return;
    }
    else {
      if (((*found)->UU.irec = ftell( cptr->stream )) < 0 ) {
	fprintf( stderr, "ADWRITE : error in ftell = %i\n"
		, (*found)->UU.irec );
	return;
      }
    }
    
  }
  else {
    ierr = fseek( cptr->stream, (*found)->UU.irec, 0 );
    if (ierr != 0) {
      fprintf( stderr, "ADWRITE : error in fseek = %i\n", ierr );
      return;
    }
  }

#ifdef DEBUG
  printf("===== tree output =========\n");
  twalk((Anyptr)(cptr->root), (Action) print_node);
  printf("===========================\n");
#endif

  fwrite( (char *) ptr, (int) *size, (int) *items, cptr->stream );

}



/***********************************************************/
/*          ADREAD                                         */
/***********************************************************/
Void
ADREAD( tfile, len, ptr, size, items, ikey )
     char    tfile[];
     FORINT  *len;
     Void    *ptr;
     FORINT  *size;
     FORINT  *items;
     FORINT  *ikey;
{
  int         ierr;
  struct code *cptr;
  struct node **found;
  int         i;
  char        tape[MAXTAPE];

  for( i=0; i<*len; i++) tape[i] = tfile[i]; tape[*len] = '\0';
  cptr = getcode(tape);

  nodeptr->key = *ikey;

#ifdef DEBUG
  printf("===== tree read output =========\n");
  twalk((Anyptr)(cptr->root), (Action) print_node);
  printf("================================\n");
#endif

  found = (struct node **)
    tsearch((Anyptr) nodeptr, (Anyptr *) &(cptr->root), (Comp) node_compare );

  if (*found == nodeptr) {
    fprintf( stderr, "ADREAD : record not found\n" );
    fprintf( stderr, " key = %i\n", nodeptr->key );
  }
  else {
    ierr = fseek( cptr->stream, (*found)->UU.irec, 0 );
    if (ierr != 0) {
      fprintf( stderr, "ADREAD : error in fseek = %i\n", ierr );
      return;
    }
    else {
      fread( (char *) ptr, (int) *size, (int) *items, cptr->stream );
    }
  }
}



/***********************************************************/
/*          MEMO_COMPARE                                   */
/***********************************************************/
Inline
Static int
memo_compare(node1, node2)
     struct memo *node1, *node2;
{
  return strcmp( node1->name, node2->name );
}

/***********************************************************/
/*          GETMEMO                                        */
/***********************************************************/
Inline
Static struct
memo *getmemo( s )
     char *s;
{
  struct memo **found;

  memoptr->name = s;
  found = (struct memo **)
    tsearch( (Anyptr) memoptr, (Anyptr *) &memo_root, (Comp) memo_compare );
  if (*found == memoptr) {
    (*found)->root   = NULL;

    if ((memoptr->name = (char *) malloc( strlen(s)+1 )) == NULL)
      fprintf( stderr, "getmemo : malloc failed\n" );
    strcpy(memoptr->name,s);

    if ((memoptr = (struct memo *) malloc( sizeof(struct memo) )) == NULL)
      fprintf( stderr, "getmemo : malloc failed\n" );
  }
  return (*found);
}


/***********************************************************/
/*          ADSTORE                                        */
/***********************************************************/
Void
ADSTORE( tfile, len, ptr, size, items, ikey )
     char    tfile[];
     FORINT  *len;
     char    *ptr;
     FORINT  *size;
     FORINT  *items;
     FORINT  *ikey;
{
  struct memo *cptr;
  struct node **found;
  FORINT      i;
  char        tape[MAXTAPE];

  for( i=0; i<*len; i++) tape[i] = tfile[i]; tape[*len] = '\0';
  cptr = getmemo( (char*) tape);

  nodeptr->key = *ikey;
 
  found = (struct node **)
    tsearch((Anyptr) nodeptr, (Anyptr *) &(cptr->root), (Comp) node_compare );

  if (*found == nodeptr) {    /* node was inserted, so get a new one */
    if ((nodeptr = (struct node *) malloc( sizeof(struct node) )) == NULL)
      fprintf( stderr, "ADSTORE : malloc failed\n" );
    if (((*found)->UU.memo = (char *) malloc( (*size)*(*items) )) == NULL)
      fprintf( stderr, "ADSTORE : malloc failed\n" );
  }

#ifdef DEBUG
  printf("===== tree output =========\n");
  twalk((Anyptr)(cptr->root), (Action) print_node);
  printf("===========================\n");
#endif

  memcpy( (Void *) (*found)->UU.memo, (Const Void *) ptr
	, (size_t) (*size)*(*items) );
}


/***********************************************************/
/*          ADRESTO                                        */
/***********************************************************/
Void
ADRESTO( tfile, len, ptr, size, items, ikey )
     char    tfile[];
     FORINT  *len;
     char    *ptr;
     FORINT  *size;
     FORINT  *items;
     FORINT  *ikey;
{
  struct memo *cptr;
  struct node **found;
  FORINT      i;
  char        tape[MAXTAPE];

  for( i=0; i<*len; i++) tape[i] = tfile[i]; tape[*len] = '\0';
  cptr = getmemo( (char*) tape);
  if (cptr == NULL) {
    fprintf( stderr, "ADRESTO : tape not found\n" );
    return;
  }

  nodeptr->key = *ikey;
 
  found = (struct node **)
    tsearch((Anyptr) nodeptr, (Anyptr *) &(cptr->root), (Comp) node_compare );

  if (*found == nodeptr) {    /* node was inserted, so get a new one */

    fprintf( stderr, "ADRESTO : record not found\n" );
    fprintf( stderr, " key = %i\n", nodeptr->key );
  }
  else {
    memcpy( (Void *) ptr, (Const Void *) (*found)->UU.memo
	  , (size_t) (*size)*(*items) );
  }
}


/***********************************************************/
/*          ADFWALK                                        */
/***********************************************************/
Void
ADFWALK( tfile, len )
     char    tfile[];
     FORINT  *len;
{
  struct code *cptr;
  FORINT      i;
  char        tape[MAXTAPE];

  for( i=0; i<*len; i++) tape[i] = tfile[i]; tape[*len] = '\0';
  cptr = getcode( (char*) tape);
  if (cptr == NULL) {
    fprintf( stderr, "ADMWALK : tape not found\n" );
    return;
  }
  printf("===== tree output =========\n");
  twalk((Anyptr)(cptr->root), (Action) print_node);
  printf("===========================\n");
}


/***********************************************************/
/*          ADMWALK                                        */
/***********************************************************/
Void
ADMWALK( tfile, len )
     char    tfile[];
     FORINT  *len;
{
  struct memo *cptr;
  FORINT      i;
  char        tape[MAXTAPE];

  for( i=0; i<*len; i++) tape[i] = tfile[i]; tape[*len] = '\0';
  cptr = getmemo( (char*) tape);
  if (cptr == NULL) {
    fprintf( stderr, "ADMWALK : tape not found\n" );
    return;
  }
  printf("===== tree output =========\n");
  twalk((Anyptr)(cptr->root), (Action) print_node);
  printf("===========================\n");
}
