      subroutine jactjac( n, x, m, vin, vout )
C*******************************************************
C computes the product of tranposed jacobian
C times jacobian times vector
C for a given vector valued function
C if variables are restored in adfunc
C then func had to be called to store these variables
C
C input  :
C      n : number of independent variables
C      x : independent variables
C      m : number of dependent variables
C    vin : arbitrary vector of pertubations
C
C output :
C   vout : result of transposed jacobian times jacobian times vin
C
C*******************************************************
      implicit none

c========================================
c define arguments
c========================================
      integer n, m
      REAL    x(n), vin(n), vout(n)

c========================================
c define local variables
c========================================
      integer i

#if   defined(DYNAMIC)
      REAL    y(m)
      REAL    vaux(m)

#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      REAL    y(1)
      pointer (py,y)
      REAL    vaux(1)
      pointer (pvaux,vaux)

#else
      integer mmax
      parameter( mmax = MAX_DEPEND )
      REAL    y(mmax)
      REAL    vaux(mmax)
#endif

c----------------------------------------
c allocate memory if non dynamic compiler
c----------------------------------------
#if   defined(DYNAMIC)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      call myalloc( py   , m*REAL_BYTE )
      call myalloc( pvaux, m*REAL_BYTE )
#endif

c----------------------------------------
c compute jacobian times vector vin
c----------------------------------------
      call g_func( n, x, m, y, vin, vaux )

c----------------------------------------
c compute transposed jacobian times vaux
c use -pure to generate adfunc
c----------------------------------------
      do i = 1,n
        vout(i) = 0.0
      end do
      call adfunc( n, x, m, vout, vaux )

c----------------------------------------
c free dynamic memory
c----------------------------------------
#if   defined(DYNAMIC)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      call myfree( py    )
      call myfree( pvaux )
#endif

      end
