#! /bin/csh
#********************************************************
#
#   Copyright (c) 1995-2000  Ralf Giering
#
# purpose:      compile and link the output of TAMC
# by:		Ralf Giering
# written:	 7.12.1996
# last changes	 21.7.1998
#
# usage : tamlink  [OPTTIONS] [--] <filenames>
# Input :
#   filename_ad.f  : adjoint routines
#   filename_ftl.f : tangent linear routines
#   filename.f     : original routines
#
# Attention:
# /usr/5bin/echo	: on sun
# \echo			: on cray
#********************************************************
# undocumented:
#set USAGE[]	= "     -svd \n"
#set USAGE[]	= "     -hessian        compute hessian\n"
#set USAGE[]	= "	-optihess	compute hessian at minimum\n"
#********************************************************
set HOMEDIR	= --HOMEDIR--
set NECHO	= --ECHO--

#set FC		= f90
set UTIL	= util
set LIBRARY	= c_cgopt
set MAIN	= prgopti.o
set OPTIONS	=
set LIBS	=
set FLAGS	=
set SRC		=
set FILES	=

set errflg	= 0
set use_reverse	= 0
set use_forward	= 0
set use_orginal	= 1

set USAGE0	= (1 2 3 4 5 6 7 8 9 10 11)
set USAGE1	= (1 2 3 4 5 6 7 8 9)
set USAGE2	= (1 2 3 4 5 6 7 8)

set USAGE0[1]	= "usage : `basename ${0}` [OPTIONS] [--] <filenames>\n\n"
set USAGE0[2]	= " OPTIONS\n"
set USAGE0[3]	= "   scalar valued functions:\n"
set USAGE0[4]	= "	-cost		compute the cost function only\n"
set USAGE0[5]	= "	-check		gradient check of adjoint code\n"
set USAGE0[6]	= "	-checktl	gradient check of tangent code\n"
set USAGE0[7]	= "	-compare	compare <Ax,y> = <x,A^*y>\n"
set USAGE0[8]	= "	-tangent	run the tangent linear model\n"
set USAGE0[9]	= "	-adjoint	run the adjoint model\n"
set USAGE0[10]	= "	-time		timing statistics\n"
set USAGE0[11]	= "	-opti		optimization\n"

set USAGE1[1]	= "   vector valued functions:\n"
set USAGE1[2]	= "	-function      	compute the function only\n"
set USAGE1[3]	= "	-forward	compute derivatives in forward mode\n"
set USAGE1[4]	= "	-reverse	compute derivatives in reverse mode\n"
set USAGE1[5]	= "	-jacobian	compute full Jacobian (reverse and forward mode)\n\n"
set USAGE1[6]	= "   common options:\n"
set USAGE1[7]	= "	-optlib	<lib>	specify optimization library [c_cgopt]\n"
set USAGE1[8]	= " 	             	   c_e04dge c_e04uce c_lsopt c_m1qn3 c_umcgg\n"
#set USAGE1[9]	= "	-fc <cmd>	specify fortran compiler command\n"

set USAGE2[1]	= "	-l and -L options will be passed to linker\n"
set USAGE2[2]	= "	all other options will be passed to Fortran-compiler\n\n"
set USAGE2[3]	= " input:\n"
set USAGE2[4]	= "	filename.f\n"
set USAGE2[5]	= "	filename_ad.f\n"
set USAGE2[6]	= "	filename_ftl.f\n\n"
set USAGE2[7]	= " output:\n"
set USAGE2[8]	= "	filename	(first name of list, without extension)\n"

#------------------------------------------
# check options
#------------------------------------------
if ($#argv == 0) then
	${NECHO} "\n${USAGE0}"
	${NECHO} "${USAGE1}"
	${NECHO} "${USAGE2}"
	exit 1
endif

while ("X$argv[1]" != "X--")
	switch ($argv[1])
#------------------------------------------
# scalar valued function options
#------------------------------------------
	case -help:
		set FILES	=
		set argv[1]	= --
		set errflg	= 1
		breaksw
	case -cost:
		set MAIN	= prgcost.o
		set use_orginal	= 1
		shift argv
		breaksw
	case -check:
		set MAIN	= prgcheck.o
		set use_reverse	= 1
 		set use_orginal = 1
		shift argv
		breaksw
	case -checktl:
		set MAIN	= prgchecktl.o
 		set use_forward	= 1
 		set use_orginal = 1
		shift argv
		breaksw
        case -tangent:
                set MAIN        = prgtangent.o
                set use_forward = 1
                shift argv
                breaksw
        case -adjoint:
                set MAIN        = prgadjoint.o
                set use_reverse = 1
                shift argv
                breaksw
	case -time:
		set MAIN	= prgtime.o
		set use_reverse	= 1
		set use_forward	= 1
		shift argv
		breaksw
	case -opti:
		set MAIN	= prgopti.o
		set use_reverse	= 1
		shift argv
		breaksw

#------------------------------------------
# vector valued function options
#------------------------------------------
	case -function:
		set MAIN	= prgfunction.o
		set use_orginal	= 1
		shift argv
		breaksw
        case -forward:
                set MAIN        = prgforward.o
                set use_forward = 1
                shift argv
                breaksw
        case -reverse:
                set MAIN        = prgreverse.o
                set use_reverse = 1
                shift argv
                breaksw
        case -compare:
                set MAIN        = prgcompare.o
                set use_reverse = 1
                set use_forward = 1
                shift argv
                breaksw
	case -jacobian:
		set MAIN	= prgjacob.o
		set use_reverse	= 1
		set use_forward	= 1
		shift argv
		breaksw

#------------------------------------------
# undocumented options
#------------------------------------------
	case -svd:
		set MAIN	= prgsvd.o
		set LIBRARY	= c_f02fje
		set use_reverse	= 1
		set use_forward	= 1
		shift argv
		breaksw
	case -hessian:
		set MAIN	= prghess.o
		set use_reverse	= 1
		shift argv
		breaksw
	case -optihess:
		set MAIN	= prgophe.o
		set use_reverse	= 1
		shift argv
		breaksw

#------------------------------------------
# common options
#------------------------------------------
	case -optlib:
		shift argv
		set LIBRARY	= $argv[1]
		shift argv
		breaksw
#	case -fc:
#		shift argv
#		set FC		= $argv[1]
#		shift argv
#		breaksw
	case -L:
		shift argv
		set OPTIONS	= "${OPTIONS} -L $argv[1]"
		shift argv
		breaksw
	case -L*:
		set OPTIONS	= "${OPTIONS} $argv[1]"
		shift argv
		breaksw
	case -l:
		shift argv
		set LIBS	= "${LIBS} -l $argv[1]"
		shift argv
		breaksw
	case -l*:
		set LIBS	= "${LIBS} $argv[1]"
		shift argv
		breaksw
	case -*:
		set FLAGS	= "${FLAGS} $argv[1]"
		shift argv
		breaksw
	default:
		set FILES	= "$*"
		set argv	= --
		breaksw
	endsw
end

shift argv
set FILES	= "${FILES} $*"

#------------------------------------------
# write usage if wrong options
#------------------------------------------
if ( ${errflg} == 1 ) then
    ${NECHO} "\n${USAGE0}"
    ${NECHO} "${USAGE1}"
    ${NECHO} "${USAGE2}"
    exit 1
endif

set errflg	= 0
set cnt	= 0
foreach i (${FILES})
    set FILE	= `basename ${i} .f`
    if (${cnt} == 0) then
      set KERN = ${FILE}
    else
      set cnt	 = 1
    endif
    if (${use_orginal} != 0) then
      if (-f ${FILE}.f) then
        set SRC = "${SRC} ${FILE}.f"
      else
        ${NECHO} " ${FILE}.f does not exist"
        set errflg	= 1
      endif
    endif
    if (${use_reverse} != 0) then
      if (-f ${FILE}_ad.f) then
        set SRC = "${SRC} ${FILE}_ad.f"
      else
        ${NECHO} " ${FILE}_ad.f does not exist"
        set errflg	= 1
      endif
    endif
    if (${use_forward} != 0) then
      if (-f ${FILE}_ftl.f) then
        set SRC = "${SRC} ${FILE}_ftl.f"
      else
        ${NECHO} " ${FILE}_ftl.f does not exist"
        set errflg	= 1
      endif
    endif
end

if (${errflg} != 0) then
  exit 1
endif

#------------------------------------------
# compile and link object code and library
#------------------------------------------
\rm -f ${KERN}

make	-f ${HOMEDIR}/Make.host			\
	-f ${HOMEDIR}/tamlink.mk		\
	EXE=${KERN}				\
	OPTIONS="${OPTIONS} -L${HOMEDIR}"	\
	ADD_FFLAGS="${FLAGS}"			\
	SRC="${HOMEDIR}/${UTIL}/${MAIN} ${SRC}"	\
	OPTLIB="-l${LIBRARY} ${LIBS}"

exit
