#include "define.h"

      SUBROUTINE OPTIMUM( N, X, OBJF, ADX )
C********************************************************************
C
C input  : N      = amount of control variables
C          X      = vector of first guess of control variables
C
C output : X      = optimal control variables
C          OBJF   = minmal costfunction
C          ADX    = gradient vector at optimum
C
C********************************************************************
      IMPLICIT NONE

C==============================================================
C declare call parameter
C==============================================================
      INTEGER N
      REAL    X(N)
      REAL    OBJF
      REAL    ADX(N)

C==============================================================
C declare local variables
C==============================================================
      EXTERNAL  QGBFUN

      INTEGER NWE04, LWE04

#if defined (DYNAMIC)
      REAL    XE04WK(13*N)
      INTEGER IE04WK(1+3*N)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      REAL    XE04WK
      INTEGER IE04WK
      POINTER (PXE04WK,XE04WK(1))
      POINTER (PIE04WK,IE04WK(1))
#else
      INTEGER NMAX
      PARAMETER( NMAX = MAX_INDEPEND )
      REAL    XE04WK(13*NMAX)
      INTEGER IE04WK(1+3*NMAX)
#endif

      INTEGER IOPTNS
      PARAMETER( IOPTNS = 7 )
      CHARACTER CC

      REAL    USER(1)
      INTEGER IUSER(1)

      INTEGER I, INFORM, IFAIL, ITERA

C======================================================
C allocate working space for E04DGE
C======================================================
      NWE04 =  13*N
      LWE04 = 1+3*N

#if defined (DYNAMIC)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      CALL MYALLOC( PXE04WK, NWE04*REAL_BYTE )
      CALL MYALLOC( PIE04WK, LWE04*INTEGER_BYTE )
#endif

C======================================================
C initialise the working space, because of 
C NAGLIB error.
C======================================================
      DO I = 1,NWE04
        XE04WK(I) = 0.0
      END DO
      DO I = 1,LWE04
        IE04WK(I)  = 0
      END DO

C======================================================
C preset the optimisation switches
C======================================================
      CALL E04DKE('FUNCTION PRECISION               = 1.E-10')
      CALL E04DKE('OPTIMALITY TOLERANCE             = 1.E-9 ')
      CALL E04DKE('PRINT LEVEL                      = 5     ')
      CALL E04DKE('VERIFY LEVEL                     = -1    ')
      CALL E04DKE('MAXIMUM STEP SIZE                = 1.E+5 ')
      CALL E04DKE('LINESEARCH TOLERANCE             = 0.95  ')
      CALL E04DKE('ESTIMATED OPTIMAL FUNCTION VALUE = 0.0   ')
      CALL E04DKE('ITERATION LIMIT                  = 20    ')

C======================================================
C read new optimisation switches from file
C======================================================
      OPEN( IOPTNS, FILE   = 'E04PARAM'
     $            , ACCESS = 'SEQUENTIAL'
     $            , FORM   = 'FORMATTED'  )
      READ( IOPTNS, '(A1)', END=8000, ERR=8000 ) CC

      REWIND IOPTNS
      CALL E04DJE( IOPTNS, INFORM )
      IF (INFORM .NE. 0)   THEN
        PRINT *,' AFTER E04DJE INFORM = ',INFORM
        STOP  'E04DJE'
      ENDIF

 8000 CONTINUE
      CLOSE ( IOPTNS )

C==============================================================
C start the optimisation
C==============================================================
      IFAIL = -1
      ITERA = 20

      CALL E04DGE( N, QGBFUN, ITERA, OBJF, ADX, X
     $           , IE04WK, XE04WK, IUSER, USER, IFAIL     )
      PRINT *,' OPTIMI : AFTER E04DGE IFAIL = ',IFAIL

C==============================================================
C end of routine
C==============================================================
      END
