#include "define.h"

      SUBROUTINE OPTIMUM( N, X, OBJF, OBJGRD )
C********************************************************************
C
C input  : N      = amount of control variables
C          X      = vector of first guess of control variables
C
C output : X      = optimal control variables
C          OBJF   = minmal costfunction
C          OBJGRD = gradient vector at optimum
C
C********************************************************************
      IMPLICIT NONE

C==============================================================
C declare call parameter
C==============================================================
      INTEGER N
      REAL    X(N)
      REAL    OBJF
      REAL    OBJGRD(N)

C==============================================================
C declare local variables
C==============================================================
      INTEGER NCLIN, NCNLN, NROWA, NROWJ, NROWR, NROWBL
      PARAMETER ( NCLIN = 0 )
      PARAMETER ( NCNLN = 0 )
      PARAMETER ( NROWA = 1 )
      PARAMETER ( NROWJ = 1 )

      REAL    A, C, CJAC, CLAMDA

      INTEGER LWORK, LIWORK

#if defined(DYNAMIC)
      REAL    WORK(20*N)
      INTEGER IWORK(3*N + NCLIN + 2*NCNLN)
      INTEGER ISTATE( N + NCLIN +   NCNLN)
      REAL    R(N*N)
      REAL    BL(N + NCLIN + NCNLN)
      REAL    BU(N + NCLIN + NCNLN)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      REAL    WORK
      INTEGER IWORK
      INTEGER ISTATE
      POINTER ( PWORK , WORK(1))
      POINTER (PIWORK ,IWORK(1))
      POINTER (PISTATE,ISTATE(1))
      REAL    R
      REAL    BL
      REAL    BU
      POINTER (PR,R(1))
      POINTER (PBL,BL(1))
      POINTER (PBU,BU(1))
#else
      INTEGER NMAX
      PARAMETER( NMAX = MAX_INDEPEND )
      REAL    WORK(20*NMAX)
      INTEGER IWORK(3*NMAX + NCLIN + 2*NCNLN)
      INTEGER ISTATE( NMAX + NCLIN +   NCNLN)
      REAL    R(NMAX*NMAX)
      REAL    BL(NMAX + NCLIN + NCNLN)
      REAL    BU(NMAX + NCLIN + NCNLN)
#endif

      INTEGER IOPTNS
      PARAMETER( IOPTNS = 7 )
      CHARACTER CC

      REAL    USER(1)
      INTEGER IUSER(1)

      EXTERNAL  QGBFUN, CONFUN

      INTEGER I, INFORM, IFAIL, ITER
      REAL    BIGBND

C======================================================
C allocate working space for E04UCE
C======================================================
      NROWR  = N
      NROWBL = N+NCLIN+NCNLN

      LWORK   = 20*N
CCCC  LWORK   = 2*N*N +  N*NCLIN + 2*N*NCNLN
CCCC $        +  20*N + 11*NCLIN +  21*NCNLN
      LIWORK = 3*N + NCLIN + 2*NCNLN


#if defined (DYNAMIC)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      CALL MYALLOC( PWORK  , LWORK *REAL_BYTE )
      CALL MYALLOC( PIWORK , LIWORK*INTEGER_BYTE )
      CALL MYALLOC( PISTATE, NROWBL*INTEGER_BYTE )
      CALL MYALLOC( PBL    , NROWBL*REAL_BYTE )
      CALL MYALLOC( PBU    , NROWBL*REAL_BYTE )
      CALL MYALLOC( PR     , NROWR*N*REAL_BYTE )
#endif

C======================================================
C initialisize the working space, because of 
C NAGLIB error.
C======================================================
      DO I = 1,LWORK
        WORK(I) = 0.0
      END DO
      DO I = 1,LIWORK
        IWORK(I)  = 0
      END DO

C======================================================
C initialisize the constraints
C======================================================
      BIGBND = 1.E+10
      DO I = 1, NROWBL
        BL(I) = -BIGBND
        BU(I) =  BIGBND
      END DO

C======================================================
C preset the optimisation switches
C======================================================
      CALL E04UEE('FUNCTION PRECISION               = 1.E-10')
      CALL E04UEE('OPTIMALITY TOLERANCE             = 1.E-9 ')
      CALL E04UEE('PRINT LEVEL                      = 5     ')
      CALL E04UEE('VERIFY LEVEL                     = -1    ')
      CALL E04UEE('LINESEARCH TOLERANCE             = 0.99  ')
      CALL E04UEE('ITERATION LIMIT                  = 20    ')

C======================================================
C read new optimisation switches from file
C======================================================
      OPEN( IOPTNS, FILE   = 'E04PARAM'
     $            , ACCESS = 'SEQUENTIAL'
     $            , FORM   = 'FORMATTED'  )
      READ( IOPTNS, '(A1)', END=8000 ) CC

      REWIND IOPTNS
      CALL E04UDE( IOPTNS, INFORM )
      IF (INFORM .NE. 0)   THEN
        PRINT *,' AFTER E04UDE INFORM = ',INFORM
        STOP  'E04UDE'
      ENDIF

 8000 CONTINUE
      CLOSE ( IOPTNS )

C==============================================================
C start the optimisation
C==============================================================
      IFAIL = -1
      ITER  = 10

      CALL E04UCE( N, NCLIN, NCNLN, NROWA, NROWJ, NROWR, A
     $           , BL, BU, CONFUN, QGBFUN, ITER, ISTATE, C, CJAC
     $           , CLAMDA, OBJF, OBJGRD, R, X
     $           , IWORK, LIWORK, WORK, LWORK 
     $           , IUSER, USER, IFAIL     )
      PRINT *,' OPTIMI : AFTER E04UCE IFAIL = ',IFAIL

C==============================================================
C end of routine
C==============================================================
      END




      SUBROUTINE CONFUN( MODE, NCNLN, N, NROWJ, NEEDC, X, C
     $                 , CJAC, NSTATE, IUSER, USER )

      INTEGER MODE, NCNLN, N, NROWJ, NSTATE
      INTEGER NEEDC(*), IUSER(*)
      REAL    X(N), C(*), CJAC(NROWJ,*), USER(*)

      END
