#include "define.h"

      subroutine minstab( nn, xx, meig, kiter, noits
     $                  , evec, eval, tol )
C********************************************************************
C
C********************************************************************
      implicit none

C==============================================================
C declare call parameter
C==============================================================
      integer nn, meig, kiter, noits
      REAL    xx(nn), evec(nn,kiter), eval(kiter), tol

C==============================================================
C declare local variables
C==============================================================
      integer novecs
      external xdot, image, monit
      REAL     xdot

      integer liwork
      parameter ( liwork = 1 )
      REAL    iwork(liwork)

      integer lwork
      integer lrwork

#if defined(DYNAMIC)
      REAL    work(nn/2 + 2*nn)
      INTEGER rwork(2*nn)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      REAL    work
      pointer (pwork,work(1))
      REAL    rwork
      pointer (prwork,rwork(1))
#else
      INTEGER nmax
      PARAMETER( nmax = MAX_INDEPEND )
      REAL    work(nmax/2 + 2*nmax)
      INTEGER rwork(2*nmax)
#endif

      integer i, ifail, nrx
      REAL    fc

C======================================================
C set some constants
C======================================================
      lwork  = 3*kiter + max0(kiter*kiter,2*nn)
      lrwork = 2*nn

C======================================================
C allocate working space for f02fje
C======================================================
#if defined(DYNAMIC)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      CALL MYALLOC( pwork , lwork *REAL_BYTE )
      CALL MYALLOC( prwork, lrwork*REAL_BYTE )
#endif

      do i = 1, lwork
        work(i) = 0.0
      end do

C==============================================================
C compute gradient at central point
C==============================================================
      do i = 1,nn
        rwork(i) = xx(i)
      end do

      call cstgrad( nn, rwork, fc, rwork(nn+1) )

C==============================================================
C start the optimisation
C==============================================================
      novecs = 0
      ifail  = 1
      nrx    = nn

      if (nn .lt. 1) then
        print '(a)', ' n < 1'
      end if

      if (meig .lt. 1) then
        print '(a)', ' m < 1'
      else if (meig .ge. kiter) then
        print '(a)', ' m >= kiter'
      end if

      if (kiter .gt. nn) then
        print '(a)', ' k > n'
      end if

      call F02FJE( nn, meig, kiter, noits, tol, xdot, image, monit
     $           , novecs, evec, nrx, eval, work, lwork
     $           , rwork, lrwork, iwork, liwork, ifail )


      print *,' OPTIMI : AFTER F02FJE IFAIL = ', ifail

      end




      REAL function xdot( iflag, n, z, w, rwork, lrwork, iwork, liwork )
C==============================================================
C==============================================================
      implicit none

      integer iflag, n
      REAL    z(n), w(n)

      integer lrwork, liwork
      integer iwork(liwork)
      REAL    rwork(lrwork)

      integer i
      REAL    sum

      sum = 0.0
      do i = 1,n
        sum = sum + z(i)*w(i)
      end do

      xdot = sum

      end




      subroutine monit( istate, nextit, nevals, nevecs, k, f, d )
C==============================================================
C==============================================================
      implicit none

      integer istate, nextit, nevals, nevecs, k
      REAL    f(k), d(k)

#ifdef DEBUG
      integer i

c==================================================
c print debug info
c==================================================
      print '(a)', ' ========= monit ========'
      print '(a,x,i5)', ' istate = ', istate
      print '(a,x,i5)', ' nextit = ', nextit
      print '(a,x,i5)', ' nevals = ', nevals
      print '(a,x,i5)', ' nevecs = ', nevecs

      print '(a)', ' errors       appr. eigenvalue'
      do i = 1,k
        print '(2(x,e12.6))', f(i), d(i)
      end do
#endif
      end

      

      subroutine image( iflag, n, z, w, rwork, lrwork, iwork, liwork )
C==============================================================
C==============================================================
      implicit none

      integer iflag, n
      REAL    z(n), w(n)

      integer lrwork, liwork
      integer iwork(liwork)
      REAL    rwork(lrwork)
      REAL    fc

#ifdef DEBUG
      integer i
      REAL    x1, x2
#endif

      call hessvec( n, rwork(1), fc, rwork(n+1), w, z )

#ifdef DEBUG
c==================================================
c print debug info
c==================================================
      print '(a)', '  i,        vin,       vout'
      x1 = 0.0
      x2 = 0.0
      do i = 1,n
        print '(x,i5,2(x,e12.6))', i, z(i), w(i)
        x1 = x1 + z(i)*z(i)
        x2 = x2 + w(i)*w(i)
      end do
      print '(6x,3(x,e12.6))', sqrt(x1), sqrt(x2), sqrt(x2)/sqrt(x1)
#endif

      end
