#ifdef CRAY
#  define NML_OPTIM OPTIM
#else
#  define NML_OPTIM NML=OPTIM
#endif

      SUBROUTINE OPTIMUM( N, X, OBJF, ADX )
C********************************************************************
C
C input  : N      = amount of control variables
C          X      = vector of first guess of control variables
C
C output : X      = optimal control variables
C          OBJF   = minmal costfunction
C          ADX    = gradient vector at optimum
C
C********************************************************************
      IMPLICIT NONE

C==============================================================
C declare call parameter
C==============================================================
      INTEGER N
      REAL    X(N)
      REAL    OBJF
      REAL    ADX(N)

C==============================================================
C declare local variables
C==============================================================
      INTEGER IOPTNS
      PARAMETER( IOPTNS = 7 )

#if defined (DYNAMIC)
      REAL    RZ1(N)
      REAL    RZ2(N)
      REAL    RZ3(N)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      REAL    RZ1(1)
      REAL    RZ2(1)
      REAL    RZ3(1)
      POINTER (PRZ1,RZ1)
      POINTER (PRZ2,RZ2)
      POINTER (PRZ3,RZ3)
#else
      INTEGER NMAX
      PARAMETER( NMAX = MAX_INDEPEND )
      REAL   RZ1(NMAX)
      REAL   RZ2(NMAX)
      REAL   RZ3(NMAX)
#endif

      EXTERNAL  SIMUL, LSLINE1

      INTEGER IFAIL

      INTEGER NITER, NFUNC, IPRINT
      REAL    EPSF, EPSX, FMIN, EPSG
      INTEGER NUPDATE
      REAL    EPS

#ifdef USE_NAMELIST
      NAMELIST /OPTIM/ NITER, NFUNC, FMIN, IPRINT
     $               , EPSF, EPSX, EPSG
     $               , NUPDATE, EPS
#endif

C======================================================
C preset the optimisation switches
C======================================================
      NITER   = 20
      NFUNC   = 40
      FMIN    = 0.0
      IPRINT  = 10

      EPSX    = 1.E-6
      EPSG    = 1.E-6
      EPS     = -1.E-6

      NUPDATE = 5

C======================================================
C read new optimisation switches from file
C======================================================
#ifdef USE_NAMELIST
      OPEN( IOPTNS, FILE   = 'lsopt.par'
     $            , ACCESS = 'SEQUENTIAL'
     $            , FORM   = 'FORMATTED'  )

      READ( IOPTNS, NML_OPTIM, END=8000, ERR=8000 )

      IF (EPS .GT. 0.0) THEN
        EPSF = EPS
        EPSX = EPS
        EPSG = EPS
      END IF

      PRINT *, ' OPTIMI : options have been read'

 8000 CONTINUE
      CLOSE( IOPTNS )
#endif

C======================================================
C allocate working space
C======================================================
#if defined(DYNAMIC)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      CALL MYALLOC( PRZ1, N*REAL_BYTE )
      CALL MYALLOC( PRZ2, N*REAL_BYTE )
      CALL MYALLOC( PRZ3, N*REAL_BYTE )
#endif

C==============================================================
C start the optimisation
C==============================================================
      CALL LSOPTV1( N, X, OBJF, ADX, SIMUL, LSLINE1
     $            , EPSX, FMIN, EPSG, IPRINT
     $            , NITER, NFUNC, NUPDATE, RZ1, RZ2, RZ3
     $            , IFAIL )

      PRINT *,' OPTIMI : AFTER LSOPT IFAIL = ', IFAIL

C==============================================================
C end of routine
C==============================================================
      END
