#ifdef CRAY
#  define NML_OPTIM OPTIM
#else
#  define NML_OPTIM NML=OPTIM
#endif

#if (REAL_BYTE == 8)
# define M1QN3    N1QN3
# define CTONBE   DTONBE
# define CTCABE   DTCABE
# define EUCLID   DUCLID
#endif

      SUBROUTINE OPTIMUM( N, X, OBJF, ADX )
C********************************************************************
C
C input  : N      = amount of control variables
C          X      = vector of first guess of control variables
C
C output : X      = optimal control variables
C          OBJF   = minmal costfunction
C          ADX    = gradient vector at optimum
C
C********************************************************************
      IMPLICIT NONE

C==============================================================
C declare call parameter
C==============================================================
      INTEGER N
      REAL    X(N)
      REAL    OBJF
      REAL    ADX(N)

C==============================================================
C declare local variables
C==============================================================
      INTEGER IOPTNS
      PARAMETER( IOPTNS = 7 )

      INTEGER NRZ

#if defined(DYNAMIC)
      INTEGER NUPMAX
      PARAMETER( NUPMAX = 20 )
      REAL    RZ(4*N + NUPMAX*(2*N+1))
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      REAL    RZ
      POINTER (PRZ,RZ(1))
#else
      INTEGER NMAX
      PARAMETER( NMAX = MAX_INDEPEND )
      INTEGER NUPMAX
      PARAMETER( NUPMAX = 20 )
      REAL    RZ(4*NMAX + NUPMAX*(2*NMAX+1))
#endif

      INTEGER IZ(5), IZS
      REAL    RZS
      DOUBLE PRECISION DZS

      EXTERNAL  SIMUL, EUCLID, CTONBE, CTCABE

      INTEGER NUPDATE, IPRINT, INDIC, NITER, NFUNC
      INTEGER MODE
      REAL    EPSX, DF1, EPSG

      REAL    FMIN, EPSF, EPS

#ifdef USE_NAMELIST
      NAMELIST /OPTIM/ NITER, NFUNC, FMIN, IPRINT
     $               , EPSF, EPSX, EPSG
     $               , NUPDATE, EPS
#endif

C==============================================================
C check the cost function
C==============================================================
      INDIC = 4
      CALL SIMUL( INDIC, N, X, OBJF, ADX, IZS, RZS, DZS )

C======================================================
C preset the optimisation switches
C======================================================
      NITER   = 20
      NFUNC   = 40
      FMIN    = 0.0
      IPRINT  = 3

      EPSX    = 1.E-6
      EPSG    = 1.E-6

      NUPDATE = 5

C======================================================
C read new optimisation switches from file
C======================================================
#ifdef USE_NAMELIST
      OPEN( IOPTNS, FILE   = 'm1qn3.par'
     $            , ACCESS = 'SEQUENTIAL'
     $            , FORM   = 'FORMATTED'  )

      READ( IOPTNS, NML_OPTIM, END=8000, ERR=8000 )

      IF (EPS .GT. 0.0) THEN
        EPSF = EPS
        EPSX = EPS
        EPSG = EPS
      END IF

      PRINT *, ' M1QN3 : options have been read'

 8000 CONTINUE
      CLOSE ( IOPTNS )
#endif

C======================================================
C allocate working space for M1QN3
C======================================================
      NRZ = 4*N + NUPDATE*(2*N+1)

#if defined(DYNAMIC)
      IF (NUPDATE .GT. NUPMAX) THEN
         PRINT '(A)', 'CALL_M1QN3 : to many update vectors'
         STOP 'ERROR'
      ENDIF
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      CALL MYALLOC( PRZ, NRZ*REAL_BYTE )
#else
      IF (NUPDATE .GT. NUPMAX) THEN
         PRINT '(A)', 'CALL_M1QN3 : to many update vectors'
         STOP 'ERROR'
      ENDIF
#endif

C==============================================================
C start the optimisation
C==============================================================
      DF1  = OBJF - FMIN
      MODE = 0
      CALL M1QN3( SIMUL, EUCLID, CTONBE, CTCABE
     $          , N, X, OBJF, ADX, EPSX, DF1
     $          , EPSG, IPRINT, 6, MODE, NITER, NFUNC
     $          , IZ, RZ, NRZ, IZS, RZS, DZS )

      PRINT *,' OPTIMI : AFTER M1QN3 MODE = ', MODE

C==============================================================
C end of routine
C==============================================================
      END
