#ifdef CRAY
#  define NML_OPTIM OPTIM
#else
#  define NML_OPTIM NML=OPTIM
#endif

#if (REAL_BYTE == 8)
#ifndef CRAY
# define UMCGG  DUMCGG
#endif
#endif

      SUBROUTINE OPTIMUM( N, X, FVALUE, ADX )
C********************************************************************
C
C input  : N      = amount of control variables
C          X      = vector of first guess of control variables
C
C output : X      = optimal control variables
C          FVALUE = minmal costfunction
C          ADX    = gradient vector at optimum
C
C********************************************************************
      IMPLICIT NONE

C==============================================================
C declare call parameter
C==============================================================
      INTEGER N
      REAL    X(N)
      REAL    FVALUE
      REAL    ADX(N)

C==============================================================
C declare local variables
C==============================================================
      INTEGER IOPTNS
      PARAMETER( IOPTNS = 7 )

      EXTERNAL  MODEL, ADM1OD, UMCGG

      INTEGER NWORK

#if defined(DYNAMIC)
      REAL   XOPT(N)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      REAL    XOPT
      POINTER (PWORK,XOPT(1))
#else
      INTEGER NMAX
      PARAMETER( NMAX = MAX_INDEPEND )
      REAL   XOPT(NMAX)
#endif

      INTEGER NSIM, I
      REAL    GRADTL, DFPRED

#ifdef USE_NAMELIST
      NAMELIST /OPTIM/ NSIM, GRADTL, DFPRED
#endif

C======================================================
C allocate working space for UMCGG
C======================================================
      NWORK = N

#if defined (DYNAMIC)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      CALL MYALLOC( PWORK, NWORK*REAL_BYTE )
#endif

C======================================================
C preset the optimisation switches
C NSIM   : maximum number of function evaluations
C GRADTL  : Convergence criteria > norm of Gradient
C DFPRED  : expected reduction in the function
C======================================================
      NSIM  = 20
      GRADTL = 1.E-8
      DFPRED = 0.05

C======================================================
C read new optimisation switches from file
C======================================================
#ifdef USE_NAMELIST
      OPEN( IOPTNS, FILE   = 'umcgg.par'
     $            , ACCESS = 'SEQUENTIAL'
     $            , FORM   = 'FORMATTED'  )

      READ( IOPTNS, NML_OPTIM, END=8000, ERR=8000 )

      PRINT *, ' OPTIMI : options have been read'

 8000 CONTINUE
      CLOSE( IOPTNS )
#endif

C==============================================================
C start the optimisation
C==============================================================
      CALL UMCGG( MODEL, ADM1OD, N, X, GRADTL, NSIM, DFPRED
     $          , XOPT, ADX, FVALUE )

C==============================================================
C end of routine
C==============================================================
      DO I = 1,N
        X(I) = XOPT(I)
      END DO

      END



C===============================================
C this subroutine computes only the gradient
C use tamc -pure -reverse
C to generate subroutine ADMODEL
C===============================================
      SUBROUTINE ADM1OD( N, X, ADX )

      IMPLICIT NONE

      INTEGER N
      REAL    X(N), ADX(N)

      INTEGER I
      REAL    FC, ADFC

C===============================================
C adjoint model
C===============================================
      DO I = 1,N
        ADX(I) = 0.0
      END DO
      ADFC = 1.0

      CALL ADMODEL( N, X, FC, ADX, ADFC )

      END
