#include "define.h"
 
      PROGRAM MAIN
C*************************************************************
      implicit none

c=========================================
c declaration
c=========================================
      integer n

C==============================================================
C get the number of control variables
C==============================================================
      call numbmod( n )

c-----------------------------------------
c call the subroutine
c-----------------------------------------
      call grcheck( n )

      end


      SUBROUTINE GRCHECK( N )
C**********************************************************************
C checks the gradient computed by the adjoint code
C by comparision with differential quotients
C**********************************************************************
      IMPLICIT NONE

      INTEGER N

      REAL    ADFC

#if defined(DYNAMIC)
      REAL   X(N)
      REAL   ADX(N)
      REAL   GFD(N)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      REAL    X, ADX, GFD
      POINTER (PX,X(1))
      POINTER (PADX,ADX(1))
      POINTER (PGFD,GFD(1))
#else
      INTEGER NMAX
      PARAMETER( NMAX = MAX_INDEPEND )
      REAL   X(NMAX)
      REAL   ADX(NMAX)
      REAL   GFD(NMAX)
#endif

C====================================
C declare local variables
C====================================
      REAL        EPSMACH
      PARAMETER ( EPSMACH = 1.E-20 )

      REAL        EPSBEG
      PARAMETER ( EPSBEG = 1.E-10 )

      INTEGER I
      REAL    FC, FC1, XMEMO, REL, FEDIFF, XMAX

      REAL    EPS
      INTEGER NBEG, NEND, NSTEP
      NAMELIST /CHECK/ EPS, NBEG, NEND, NSTEP

      CHARACTER*(*) CFILE
      PARAMETER ( CFILE = 'check.par')

C==============================================================
C allocate memory for the control variables
C and the gradient vectors
C==============================================================
#if defined(DYNAMIC)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      CALL MYALLOC( PX  , N*REAL_BYTE )
      CALL MYALLOC( PADX, N*REAL_BYTE )
      CALL MYALLOC( PGFD, N*REAL_BYTE )
#endif

C==============================================================
C initialisize the model
C and set the first guess of the control variables
C==============================================================
      CALL INITMOD( N, X )

C==============================================================
C set default parameter
C read parameter von file and check
C==============================================================
      EPS   = EPSBEG
      NBEG  = 1
      NEND  = N
      NSTEP = 1

#ifdef USE_NAMELIST
      OPEN (UNIT=87, FILE=CFILE )
      READ (UNIT=87, NML_CHECK, END=100, ERR=100 )
      PRINT '(A)',' parameters for gradient check read'
 100  CLOSE(UNIT=87)
#endif

      NBEG = MAX(1,NBEG)
      NBEG = MIN(N,NBEG)
      NEND = MAX(1,NEND)
      NEND = MIN(N,NEND)

C==============================================================
C check every component of the gradient vector
C==============================================================
      PRINT '(A)', '============================================'
      PRINT '(A,E9.3)', ' CHECK OF GRADIENTS USING EPS = ', EPS
      PRINT '(A)', '============================================'
      PRINT *
      PRINT '(2A)', '     I         X(I)  FC1-FC2/EPS'
     $     , '     GRAD(FC) RELATIVE ERR'

      DO I = 1, N
        ADX(I) = 0.0
      END DO
      ADFC = 1.0
      CALL ADMODEL( N, X, FC, ADX, ADFC )

      DO I = NBEG, NEND, NSTEP
         XMEMO = X(I)
         X(I) = X(I) + EPS
         CALL MODEL( N, X, FC1 )
         FEDIFF = FC1 - FC
         IF (ABS(FEDIFF) .LT. EPSMACH) THEN
           GFD(I) = 0.0
         ELSE
           GFD(I) = FEDIFF / EPS
         END IF
         X(I) = XMEMO

         rel = abs(gfd(i)-adx(i))
         xmax = max( abs(gfd(i)), abs(adx(i)) )
         if (xmax .eq. 0.0) then
            rel = rel * 1.e+4
         else
            rel = rel / xmax
         end if
         if (rel .lt. 1.e-3) then
            print '(i6,4(1x,e12.6)  )',i,x(i),gfd(i),adx(i),rel
         else
            print '(i6,4(1x,e12.6),a)',i,x(i),gfd(i),adx(i),rel,' BAD?'
         endif
      END DO

      END
