#include "define.h"

      PROGRAM MAIN
C*************************************************************
      implicit none

c=========================================
c declaration
c=========================================
      integer n

C==============================================================
C get the number of control variables
C==============================================================
      call numbmod( n )

c-----------------------------------------
c call the subroutine
c-----------------------------------------
      call checktl( n )

      end


      SUBROUTINE CHECKTL( N )
C**********************************************************************
C checks the gradient computed by tangent linear code
C by comparision with differential quotients
C**********************************************************************
      IMPLICIT NONE

      INTEGER N

      REAL    G_FC

#if defined(DYNAMIC)
      REAL   X(N)
      REAL   G_X(N)
      REAL   GFD(N)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      REAL    X, G_X, GFD
      POINTER (PX,X(1))
      POINTER (PG_X,G_X(1))
      POINTER (PGFD,GFD(1))
#else
      INTEGER NMAX
      PARAMETER( NMAX = MAX_INDEPEND )
      REAL   X(NMAX)
      REAL   G_X(NMAX)
      REAL   GFD(NMAX)
#endif

C====================================
C declare local variables
C====================================
      REAL    EPSMACH
      PARAMETER ( EPSMACH = 1.E-20 )

      REAL    EPSBEG
      PARAMETER ( EPSBEG = 1.E-10 )

      REAL    TEPS
      PARAMETER ( TEPS = 1.E-4 )

      INTEGER I
      REAL    FC, FC1, FCH, XMEMO, REL, FEDIFF, XMAX

      REAL    EPS
      INTEGER NBEG, NEND, NSTEP

#ifdef USE_NAMELIST
      NAMELIST /CHECK/ EPS, NBEG, NEND, NSTEP
#endif

      CHARACTER*(*) CFILE
      PARAMETER ( CFILE = 'check.par')

C==============================================================
C allocate memory for the control variables
C and the gradient vector
C==============================================================
#if defined(DYNAMIC)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      CALL MYALLOC( PX  , N*REAL_BYTE )
      CALL MYALLOC( PG_X, N*REAL_BYTE )
      CALL MYALLOC( PGFD, N*REAL_BYTE )
#endif

C==============================================================
C initialisize the model
C and set the first guess of the control variables
C==============================================================
      CALL INITMOD( N, X )

C==============================================================
C set default parameter
C read parameter von file and check
C==============================================================
      EPS   = EPSBEG
      NBEG  = 1
      NEND  = N
      NSTEP = 1

#ifdef USE_NAMELIST
      OPEN (UNIT=87, FILE=CFILE )
      READ (UNIT=87, NML_CHECK, END=100, ERR=100 )
      PRINT '(A)',' parameters for gradient check read'
 100  CLOSE(UNIT=87)
#endif

      NBEG = MAX(1,NBEG)
      NBEG = MIN(N,NBEG)
      NEND = MAX(1,NEND)
      NEND = MIN(N,NEND)

C==============================================================
C check every component of the gradient vector
C==============================================================
      PRINT '(A)', '==================================================='
      PRINT '(A,E9.3)', ' CHECK OF PERTUBATIONS USING EPS = ', EPS
      PRINT '(A)', '==================================================='
      PRINT *
      PRINT '(2A)', '     I         X(I)  FC1-FC2/EPS'
     $     , '     GRAD(FC)   DIFFERENCE '

      CALL MODEL( N, X, FC )

      DO I = 1, N
         G_X(I) = 0.0
      END DO

      DO I = NBEG, NEND, NSTEP
         G_X(I) = TEPS
         CALL G_MODEL( N, X, FCH, G_X, G_FC )
         G_X(I) = 0.0
         G_FC   = G_FC/TEPS

         XMEMO = X(I)
         X(I) = X(I) + EPS
         CALL MODEL( N, X, FC1 )
         X(I) = XMEMO

         FEDIFF = FC1 - FC
         IF (ABS(FEDIFF) .LT. EPSMACH) THEN
           GFD(I) = 0.0
         ELSE
           GFD(I) = FEDIFF / EPS
         END IF

         rel = abs(gfd(i)-g_fc)
         xmax = max( abs(gfd(i)), abs(g_fc) )
         if (xmax .eq. 0.0) then
            rel = rel * 1.e+4
         else
            rel = rel / xmax
         end if
         if (rel .lt. 1.e-3) then
            print '(i6,4(1x,e12.6)  )',i,x(i),gfd(i),g_fc,rel
         else
            print '(i6,4(1x,e12.6),a)',i,x(i),gfd(i),g_fc,rel,' BAD?'
         endif

      END DO

      END
