#include "define.h"

      PROGRAM MAIN
C*************************************************************
      implicit none

c=========================================
c declaration
c=========================================
      integer n

C==============================================================
C get the number of control variables
C==============================================================
      call numbmod( n )

c-----------------------------------------
c call the subroutine
c-----------------------------------------
      call compare( n )

      end


      SUBROUTINE COMPARE( N )
C**********************************************************************
C checks the output computed by the reverse and forward mode
C**********************************************************************
      IMPLICIT NONE

      INTEGER N
      REAL    ADFC

#if defined(DYNAMIC)
      REAL    X(N)
      REAL    ADX(N)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      REAL    X, ADX
      POINTER (PX,X(1))
      POINTER (PADX,ADX(1))
#else
      INTEGER NMAX
      PARAMETER( NMAX = MAX_INDEPEND )
      REAL    X(NMAX)
      REAL    ADX(NMAX)
#endif

C====================================
C local variables
C====================================
      INTEGER I
      REAL    FC, FCR, FCF, TLFC, SUM

C==============================================================
C allocate memory for the control variables
C and the gradient vector
C==============================================================
#if defined(DYNAMIC)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      CALL MYALLOC( PX  , N*REAL_BYTE )
      CALL MYALLOC( PADX, N*REAL_BYTE )
#endif

C==============================================================
C initialisize the model
C and set the first guess of the control variables
C==============================================================
      CALL INITMOD( N, X )

C==============================================================
C check every component of the gradient vector
C==============================================================
      PRINT *
      PRINT '(A)', '============================================'
      PRINT '(A)', '     CHECK OF REVERSE AND FORWARD MODE'
      PRINT '(A)', '  USING DEFINITION OF THE ADJOINT OPERATOR'
      PRINT '(A)', '          < Ax, y > := < x, A*y >'
      PRINT '(A)', '============================================'
      PRINT *

      CALL MODEL( N, X, FC )

      DO I = 1, N
        ADX(I) = 0.0
      END DO
      ADFC = 1.0
      CALL ADMODEL( N, X, FCR, ADX, ADFC )

      SUM = 0.0
      DO I = 1,N
        SUM = SUM + ADX(I)*ADX(I)
      ENDDO

      PRINT '(A,E14.8)', ' <   A* x, A* x > = ', SUM

      TLFC = 0.0
      CALL G_MODEL( N, X, FCF, ADX, TLFC )

      PRINT '(A,E14.8)', ' < A A* x,    x > = ', TLFC

      PRINT *
      PRINT '(A,E14.8)', ' original F(X) = ', FC
      PRINT '(A,E14.8)', ' reverse  F(X) = ', FCR
      PRINT '(A,E14.8)', ' forward  F(X) = ', FCF
      PRINT *

      END
