#include "define.h"

      PROGRAM MAIN
C*************************************************************
      implicit none

c=========================================
c declaration
c=========================================
      integer n

C==============================================================
C get the number of control variables
C==============================================================
      call numbmod( n )

c-----------------------------------------
c call the subroutine
c-----------------------------------------
      call ophess( n )

      end


      SUBROUTINE OPHESS( N )
C==============================================================
C declaration
C==============================================================
      IMPLICIT NONE

      INTEGER N

#if defined(DYNAMIC)
      REAL   X(N)
      REAL   ADX(N)
      REAL   XH(N*N)
      REAL   EIGVAL(N)
      REAL   EIGVEC(N*N)
      REAL   WORK(N)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      REAL    X, ADX
      POINTER (PX,X(1))
      POINTER (PADX,ADX(1))
      REAL    XH
      POINTER (PXH,XH(1))
      REAL     EIGVAL, WORK, EIGVEC
      POINTER (PEIGVAL,EIGVAL(1))
      POINTER (PWORK,WORK(1))
      POINTER (PEIGVEC,EIGVEC(1,1))
#else
      INTEGER NMAX
      PARAMETER( NMAX = MAX_INDEPEND )
      REAL   X(NMAX)
      REAL   ADX(NMAX)
      REAL   XH(NMAX*NMAX)
      REAL   EIGVAL(NMAX)
      REAL   EIGVEC(NMAX*NMAX)
      REAL   WORK(NMAX)
#endif

      REAL    OBJF
      INTEGER IFAIL

C==============================================================
C allocate memory for the control variables
C and the gradient vector
C==============================================================
#if defined(DYNAMIC)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      CALL MYALLOC( PX     , N*REAL_BYTE )
      CALL MYALLOC( PADX   , N*REAL_BYTE )
      CALL MYALLOC( PEIGVAL, N*REAL_BYTE )
      CALL MYALLOC( PWORK  , N*REAL_BYTE )
      CALL MYALLOC( PEIGVEC, N*N*REAL_BYTE )
      CALL MYALLOC( PXH    , N*N*REAL_BYTE )
#endif

C==============================================================
C initialisize the model
C and set the first guess of the control variables
C==============================================================
      CALL INITMOD( N, X )

C==============================================================
C optimize the costfunction using control variables
C==============================================================
      CALL OPTIMUM( N, X, OBJF, ADX )

C==============================================================
C start of postprocessor
C==============================================================
      CALL POSTMOD( N, X, OBJF, ADX )

C===============================================
C calculate hessian matrix
C===============================================
      CALL HESSIAN( N, X, OBJF, ADX, XH )

C===============================================
C calculate eigenvectors of hessian matrix
C Attention, Eigenvalues are in reverse order!
C===============================================
      IFAIL = 0
      CALL F02ABE ( XH, N, N, EIGVAL, EIGVEC, N, WORK, IFAIL )

      IF (IFAIL.NE.0) THEN
        PRINT *,' call of F02ABF returned ifail = ', IFAIL
        STOP 'ERROR'
      ENDIF

C===============================================
C start of postprocessor
C===============================================
      CALL POSTHES( N, XH, EIGVAL, EIGVEC )

      END

