#include "define.h"

      PROGRAM MAIN
C*************************************************************
      implicit none

c=========================================
c declaration
c=========================================
      integer n, meig, kiter, noits

C==============================================================
C get the number of control variables
C==============================================================
      call numbmod( n )

c-----------------------------------------
c set some dummy values
c-----------------------------------------
      meig  = min0( n-4, 5 )
      kiter = min0( meig+5, n )
      noits = 100

c-----------------------------------------
c call the subroutine
c-----------------------------------------
      call svdata( n, meig, kiter, noits )

      end



      subroutine svdata( n, meig, kiter, noits )
C==============================================================
C declaration
C==============================================================
      IMPLICIT NONE

      INTEGER N, MEIG, KITER, NOITS

      REAL    TOL
      PARAMETER( TOL = 1.E-2)
      INTEGER MEIGF

#if defined(DYNAMIC)
      REAL   X(N)
      REAL   EIGVAL(KITER)
      REAL   EIGVEC(N*KITER)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      REAL    X
      POINTER (PX,X(1))
      REAL     EIGVAL, EIGVEC
      POINTER (PEIGVAL,EIGVAL(1))
      POINTER (PEIGVEC,EIGVEC(1,1))
#else
      INTEGER NMAX
      PARAMETER( NMAX = MAX_INDEPEND )
      REAL   X(NMAX)
      REAL   EIGVAL(NMAX)
      REAL   EIGVEC(NMAX*NMAX)
#endif

C==============================================================
C allocate memory for the control variables
C and the gradient vector
C==============================================================
#if defined(USE_POINTER) || (MAX_INDEPEND == 0)
      CALL MYALLOC( PX     , N       *REAL_BYTE )
      CALL MYALLOC( PEIGVAL, KITER   *REAL_BYTE )
      CALL MYALLOC( PEIGVEC, N*KITER *REAL_BYTE )
#endif

C==============================================================
C initialisize the model
C and set the first guess of the control variables
C==============================================================
      CALL INITMOD( N, X )

C===============================================
C calculate eigenvectors of hessian matrix
C Attention, Eigenvalues are in decreasing order!
C===============================================
      MEIGF = MEIG

      CALL MINSTAB( N, X, MEIGF, KITER, NOITS, EIGVEC, EIGVAL, TOL )

C===============================================
C start of postprocessor
C===============================================
      IF (MEIGF .NE. 0) THEN
        CALL POSTSVD( N, MEIGF, KITER, NOITS, EIGVAL, EIGVEC )
      ELSE
        PRINT '(A)', ' !!!! no eigenvectors found'
      END IF

      END


      SUBROUTINE POSTSVD( N, MEIG, KITER, NOITS, EIGVAL, EIGVEC )
      IMPLICIT NONE

      INTEGER N, MEIG, KITER, NOITS, I, J
      REAL    EIGVAL(MEIG), EIGVEC(N,MEIG)

      PRINT '(A)', '=============================='
      PRINT '(A,I5,A)', ' kiter = ', KITER, ' vectors used'
      PRINT '(A,I5,A)', ' noits = ', NOITS, ' iterations'
      PRINT '(A,I5,A)', ' the ', MEIG, ' eigenvalues are'
      PRINT '(6(X,E12.6))', (EIGVAL(J),J=1,MEIG)
      PRINT '(A)', ' and the cooresponding eigenvectors are'
      DO I = 1,N
        PRINT '(6(X,E12.6))', (EIGVEC(I,J),J=1,MEIG)
      END DO

      END
