#include "define.h"

      PROGRAM MAIN
C*************************************************************
C compute singular value decomposition of
C tranposed jacobian times jacobian
C*************************************************************
      implicit none

c=========================================
c declaration
c=========================================
      integer n, m, meig, kiter, noits

c-----------------------------------------
c get the number of independent
c and dependent variables
c-----------------------------------------
      call setfunc( n, m )

c-----------------------------------------
c set some dummy values
c-----------------------------------------
      meig  = min0( n-4, 5 )
      kiter = min0( meig+5, n )
      noits = 100

c-----------------------------------------
c call the subroutine
c-----------------------------------------
      call svdata( n, m, meig, kiter, noits )

      end



      subroutine svdata( n, m, meig, kiter, noits )
C*************************************************************
C compute meig eigenvectors
C of tranposed jacobian times jacobian
C compute product of kiter vectors simultaneously
C stop after noits iterations
C*************************************************************
      implicit none

c=========================================
c declaration
c=========================================
      integer n, m, meig, kiter, noits

#if defined(DYNAMIC)
      REAL    x(n)
      REAL    eigval(kiter)
      REAL    eigvec(n*kiter)

#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      REAL    x(1)
      pointer (px,x)
      REAL    eigval(1)
      pointer (peigval,eigval)
      REAL    eigvec(1)
      pointer (peigvec,eigvec)

#else
      integer nmax
      parameter( nmax = MAX_INDEPEND )
      REAL    x(nmax)
      REAL    eigval(nmax)
      REAL    eigvec(nmax*nmax)

#endif

      REAL    tol
      parameter (tol = 1.e-2)
      integer meigf

c-----------------------------------------
c allocate dynamic memory
c-----------------------------------------
#if defined(DYNAMIC)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      call myalloc( px     , n       *REAL_BYTE )
      call myalloc( peigval, kiter   *REAL_BYTE )
      call myalloc( peigvec, n*kiter *REAL_BYTE )
#endif

c-----------------------------------------
c initialize independent variables
c-----------------------------------------
      call initfunc( n, x )

c-----------------------------------------
c calculate eigenvectors of hessian matrix
c Attention, Eigenvalues are in decreasing order!
c-----------------------------------------
      meigf = meig

      call minstab( n, x, m, meigf, kiter, noits, eigvec, eigval, tol )

c-----------------------------------------
c call postprocessor
c-----------------------------------------
      if (meigf .ne. 0) then
         call postsvd( n, meigf, kiter, noits, eigval, eigvec )
      else
         print '(a)', ' !!!! no eigenvectors found'
      end if

      end




      SUBROUTINE POSTSVD( N, MEIG, KITER, NOITS, EIGVAL, EIGVEC )
C*************************************************************
C postprocessing
C*************************************************************
      implicit none

      INTEGER N, MEIG, KITER, NOITS, I, J
      REAL    EIGVAL(MEIG), EIGVEC(N,MEIG)

      PRINT '(A)', '=============================='
      PRINT '(A,I5,A)', ' kiter = ', KITER, ' vectors used'
      PRINT '(A,I5,A)', ' noits = ', NOITS, ' iterations'
      PRINT '(A,I5,A)', ' the ', MEIG, ' eigenvalues are'
      PRINT '(6(X,E12.6))', (EIGVAL(J),J=1,MEIG)
      PRINT '(A)', ' and the cooresponding eigenvectors are'
      DO I = 1,N
        PRINT '(6(X,E12.6))', (EIGVEC(I,J),J=1,MEIG)
      END DO

      END





      subroutine minstab( n, x, m, meig, kiter, noits
     $                  , evec, eval, tol )
C********************************************************************
C
C********************************************************************
      implicit none

C==============================================================
C declare call parameter
C==============================================================
      integer n, m, meig, kiter, noits
      REAL    x(n), evec(n,kiter), eval(kiter), tol

C==============================================================
C declare local variables
C==============================================================
      integer novecs
      external xdot, image, monit
      REAL     xdot

      integer liwork
      parameter ( liwork = 1 )
      REAL    iwork(liwork)

      integer lwork
      integer lrwork

#if defined(DYNAMIC)
CCCC  work should be 3*kiter + max0(kiter*kiter,2*n) long
      REAL    y(m)
      REAL    work(3*kiter + 2*n)
      REAL    rwork(n)

#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      REAL    y(1)
      pointer (py,y)
      REAL    work(1)
      pointer (pwork,work)
      REAL    rwork(1)
      pointer (prwork,rwork)
#else
      integer nmax
      parameter( nmax = MAX_INDEPEND )
      integer mmax
      parameter( mmax = MAX_DEPEND )

      REAL    work(nmax/2 + 2*nmax)
      integer rwork(nmax)
      REAL    y(mmax)
#endif

      integer i, ifail, nrx

C======================================================
C set some constants
C======================================================
      lwork  = 3*kiter + max0(kiter*kiter,2*n)
      lrwork = n

C======================================================
C allocate working space for f02fje
C======================================================
#if defined(DYNAMIC)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      CALL MYALLOC( py    , m     *REAL_BYTE )
      CALL MYALLOC( pwork , lwork *REAL_BYTE )
      CALL MYALLOC( prwork, lrwork*REAL_BYTE )
#endif

      do i = 1, lwork
        work(i) = 0.0
      end do

c-----------------------------------------
c save independent variables
c used in image
c-----------------------------------------
      do i = 1,n
        rwork(i) = x(i)
      end do

c-----------------------------------------
c compute function
c-----------------------------------------
      call func( n, x, m, y )

c-----------------------------------------
C start
c-----------------------------------------
      novecs = 0
      ifail  = 1
      nrx    = n

      if (n .lt. 1) then
        print '(a)', ' n < 1'
      end if

      if (meig .lt. 1) then
        print '(a)', ' m < 1'
      else if (meig .ge. kiter) then
        print '(a)', ' m >= kiter'
      end if

      if (kiter .gt. n) then
        print '(a)', ' k > n'
      end if

      call F02FJE( n, meig, kiter, noits, tol, xdot, image, monit
     $           , novecs, evec, nrx, eval, work, lwork
     $           , rwork, lrwork, iwork, liwork, ifail )


      print *,' OPTIMI : AFTER F02FJE IFAIL = ', ifail

      end






      REAL function xdot( iflag, n, z, w, rwork, lrwork, iwork, liwork )
C*************************************************************
C*************************************************************
      implicit none

      integer iflag, n
      REAL    z(n), w(n)

      integer lrwork, liwork
      integer iwork(liwork)
      REAL    rwork(lrwork)

      integer i
      REAL    sum

      sum = 0.0
      do i = 1,n
        sum = sum + z(i)*w(i)
      end do

      xdot = sum

      end




      subroutine monit( istate, nextit, nevals, nevecs, k, f, d )
C*************************************************************
C*************************************************************
      implicit none

      integer istate, nextit, nevals, nevecs, k
      REAL    f(k), d(k)

#ifdef DEBUG
      integer i

c-----------------------------------------
c print debug info
c-----------------------------------------
      print '(a)', ' ========= monit ========'
      print '(a,x,i5)', ' istate = ', istate
      print '(a,x,i5)', ' nextit = ', nextit
      print '(a,x,i5)', ' nevals = ', nevals
      print '(a,x,i5)', ' nevecs = ', nevecs

      print '(a)', ' errors       appr. eigenvalue'
      do i = 1,k
        print '(2(x,e12.6))', f(i), d(i)
      end do
#endif
      end

      

      subroutine image( iflag, n, z, w, rwork, lrwork, iwork, liwork )
C*************************************************************
C*************************************************************
      implicit none

      integer iflag, n
      REAL    z(n), w(n)

      integer lrwork, liwork
      integer iwork(liwork)
      REAL    rwork(lrwork)

#ifdef DEBUG
      integer i
      REAL    x1, x2
#endif

c-----------------------------------------
c compute transposed jacobian times
c jacobian times vector
c subroutine jactjac( n, x, m, vin, vout )
c-----------------------------------------
      call jactjac( n, rwork(1), iwork(1), z, w )

#ifdef DEBUG
c-----------------------------------------
c print debug info
c-----------------------------------------
      print '(a)', '  i,        vin,       vout'
      x1 = 0.0
      x2 = 0.0
      do i = 1,n
        print '(x,i5,2(x,e12.6))', i, z(i), w(i)
        x1 = x1 + z(i)*z(i)
        x2 = x2 + w(i)*w(i)
      end do
      print '(6x,3(x,e12.6))', sqrt(x1), sqrt(x2), sqrt(x2)/sqrt(x1)
#endif

      end
