#include "define.h"

      PROGRAM MAIN
C*************************************************************
      implicit none

c=========================================
c declaration
c=========================================
      integer n

C==============================================================
C get the number of control variables
C==============================================================
      call numbmod( n )

c-----------------------------------------
c call the subroutine
c-----------------------------------------
      call comtime( n )

      end


      SUBROUTINE COMTIME( N )

      IMPLICIT NONE

      INTEGER N
      REAL    FC, ADFC, TLFC

      INTEGER I, ITER

      REAL    TFUNC, TGRAD, TFORW, SEC1, SEC2
      REAL    T0, DT

      real*4   second
      EXTERNAL second

#if defined(DYNAMIC)
      REAL   X(N)
      REAL   ADX(N)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      REAL    X, ADX
      POINTER (PX,X(1))
      POINTER (PADX,ADX(1))
#else
      INTEGER NMAX
      PARAMETER( NMAX = MAX_INDEPEND )
      REAL   X(NMAX)
      REAL   ADX(NMAX)
#endif

      INTEGER IOPTNS
      PARAMETER( IOPTNS = 7 )
      INTEGER NITER

#ifdef USE_NAMELIST
      NAMELIST /OPTIM/ NITER
#endif


C==============================================================
C set time delay of call
C==============================================================
      T0 = second()
      DT = second() - T0

C==============================================================
C allocate memory for the control variables
C and the gradient vector
C==============================================================
#if defined(DYNAMIC)
#elif defined(USE_POINTER) || (MAX_INDEPEND == 0)
      CALL MYALLOC( PX  , N*REAL_BYTE )
      CALL MYALLOC( PADX, N*REAL_BYTE )
#endif

C==============================================================
C initialisize the model
C and set the first guess of the control variables
C==============================================================
      CALL INITMOD( N, X )

      TFUNC = 0.0
      TGRAD = 0.0

C======================================================
C preset the switches
C======================================================
      NITER   = 10

C======================================================
C read new optimisation switches from file
C======================================================
#ifdef USE_NAMELIST
      OPEN( IOPTNS, FILE   = 'time.par'
     $            , ACCESS = 'SEQUENTIAL'
     $            , FORM   = 'FORMATTED'  )

      READ( IOPTNS, NML_OPTIM, END=8000, ERR=8000 )

      PRINT *, ' PRGTIME : options have been read'

 8000 CONTINUE
      CLOSE( IOPTNS )
#endif

C===============================================
C forward model
C first call to do possible initializations
C===============================================
      CALL MODEL( N, X, FC )

      SEC1 = second()
      DO ITER = 1, NITER
         CALL MODEL( N, X, FC )
      END DO

      SEC2 = second()
      TFUNC = TFUNC + ((SEC2-SEC1) - DT)

C===============================================
C reset all adjoint variables
C===============================================
      SEC1 = second()
      DO ITER = 1, NITER
         DO I = 1, N
            ADX(I) = 0.0
         END DO
         ADFC = 1.0
         CALL ADMODEL( N, X, FC, ADX, ADFC )
      END DO
      SEC2 = second()
      TGRAD = TGRAD + ((SEC2-SEC1) - DT)

      SEC1 = second()
      DO ITER = 1, NITER
         CALL G_MODEL( N, X, FC, ADX, TLFC )
      END DO
      SEC2 = second()
      TFORW = TFORW + ((SEC2-SEC1) - DT)

C===============================================
C print quotient
C===============================================
      IF (NITER .GT. 1) THEN
        PRINT *, ' iterations = ', NITER
      ENDIF

      PRINT *
      PRINT *, '**************************************************'
      PRINT *, '  TIMING OUTPUT'
      PRINT *, '**************************************************'
      PRINT '(a,i8)', ' control parameters N         : ', N
      PRINT '(a,f12.5)', ' run time function            : ', TFUNC
      PRINT '(a,f12.5)', ' run time function + adjoint  : ', TGRAD
      PRINT '(a,f12.5)', ' run time function + t.linear : ', TFORW

      PRINT '(2a,f9.3)', ' rel. run time reverse mode'
     $     , ' (FUNC+GRAD)/FUNC          = ', TGRAD/TFUNC
      PRINT '(2a,f9.3)', ' rel. run time forward mode'
     $     , ' (FUNC+PERT)/FUNC          = ', TFORW/TFUNC
      PRINT '(2a,f9.3)', ' rel. run time             '
     $     , '(Finite Diff.)/(FUNC+GRAD) = ', (N+1)*TFUNC/TGRAD
      PRINT *, '**************************************************'
      PRINT *

      END

